/*
 * @author Walther Neuper, member of the ISAC-team, Copyright (c) 2005 by
 * Walther Neuper created Jun 18, 2005 2:56:04 PM Institute for
 * Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.TestConstants;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Context;
import isac.util.formulae.ContextProblem;
import isac.util.formulae.KEStoreKey;
import isac.util.formulae.MethodID;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;
import isac.util.formulae.ProblemID;
import isac.util.parser.FormalizationDigest;

import java.util.Vector;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Sep 28, 2004, 3:37:02 PM
 * @version 1.0
 * @see isac.functest.specify.TestSpecify for related tests
 */
public class TestSpecify extends TestCase {

    Formalization fmz_;
    MathEngine me_;

    protected void setUp() throws Exception {
        super.setUp();
        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TESTS_7A);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        fmz_ = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        me_ = MathEngine.getMathEngine();
    }

    /**
     * {\bf\UC{Having \isac{} Complete the
     * Specification}\label{SPECIFY:COMPLETE:spec_}\\} this is done with the
     * same button as 'complete CalcHead'
     * 
     * @see isac.bridge.TestSpecify#testCompleteSpecification() for how this
     *      looks like on the fron-end
     */
    public void testCompleteSpecification() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestSpecify#testCompleteSpecification");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model ...
        ch = ct.completeModel();
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Specification by completeCalcHead
        assertEquals(ch.getSpecification().toSMLString(),
                "\"e_domID\",[\"e_pblID\"],[\"e_metID\"]");
        ch = ct.completeCalcHead();
        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
                + "[\"maximum_of\",\"function\"],"
                + "[\"DiffApp\",\"max_by_calculus\"]");
        System.out.println("---END isac.bridge.TestSpecify#testCompleteSpecification");
    }

    /**
     * {\bf\UC{Having \isac{} Complete One Field of the
     * Specification}\label{SPECIFY:COMPLETE:specone}\\}
     * 
     * @see isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification() for
     *      how this looks like on the fron-end
     * @deprecated due to input by transfer from the knowledge-browsers
     */
    public void testCompleteOneFieldOfSpecification() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model ...
        ch = ct.completeModel();
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Specification
        //could be continued by nextSpecify(), i.e. by <Next>-button,
        //but here by stepwise input
        assertEquals("\"e_domID\",[\"e_pblID\"],[\"e_metID\"]", ch.getSpecification().toSMLString());
        //input the Theory ...
        ch = ct.setTheory("DiffApp.thy");
        assertEquals("\"DiffApp.thy\",[\"e_pblID\"],[\"e_metID\"]", ch.getSpecification().toSMLString());
        //input the Problem ...
        Vector<String> keylist = new Vector<>();
        keylist.add("maximum_of");
        keylist.add("function");
        ct.setProblem(new ProblemID(keylist));
        //input the Method ...
        keylist = new Vector<>();
        keylist.add("DiffApp");
        keylist.add("max_by_calculus");
        ch = ct.setMethod(new MethodID(keylist));
        //FIXXXME.WN051124 here Specify_Method causes a 'helpless' in sml !!!
        // assertEquals("\"DiffApp.thy\"," + "[\"maximum_of\",\"function\"],"
        // + "[\"DiffApp\",\"max_by_calculus\"]", ch.getSpecification()
        // .toSMLString());
        System.out.println("---END isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification");
    }

    /**
     * {\bf\UC{Having \isac{} Refine the Problem
     * Automatically}\label{SPECIFY:refine-auto}\\} test match and refine with
     * x^2 + 4*x + 5 = 2 smltest/FE-interface/interface.sml ---
     * tryMatchProblem, tryRefineProblem ---
     * 
     * @see isac.functest.problems.TestRefinement#testAutomaticallyRefinement()
     *      for how this looks like at the front-end.
     * 
     * this run requires another example !!!
     */
    public void testMatchRefine() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestSpecify#testMatchRefine");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices(TestConstants.KESTORE.XMLDATA_PATH);
        Vector<String> vec = ks.loadContent("exp", TestConstants.KESTORE.TEST_DATA.EXP_TEST_1D);
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        System.out.println(".end step_b-----");
        ICalcIterator ci = ct.getActiveFormula();
        //ci = new CalcIterator();
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the CalcHead with Descriptions by 2-times <NEXT>:
        ct.modelProblem();
        ProblemID id;
        id = new ProblemID();
        id.addString("univariate");
        id.addString("equation");

        ContextProblem context2check = new ContextProblem();
        context2check.setKEStoreKey(new KEStoreKey("pbl_equ_univ"));
        @SuppressWarnings("unused") // used in outcommented code
        Context cont2;
        //context2check.setHierarchyKey(hk);

        //trymatch ... go into the problem-hierarchy with this CalcHeads Model
        //this Match will show all items complete:

        cont2 = ci.checkContext(context2check);
        //m = ci.tryMatchProblem(id);
        /*assertEquals(m.getHeadline().toSMLString(), "??.empty");//...TODO.WN050626
        assertEquals(m.getModel().getGiven().toString(),
        "[\"correct (equality (x ^ 2 + 4 * x + 5 = 2))\", "
        + "\"correct (solveFor x)\"]");
        assertEquals(m.getModel().getFind().toString(),
        "[\"correct (solutions L)\"]");
        assertEquals(m.getStatus(), "incorrect");//because the specification is
        // missing! TODO.WN0506 correct to 'correct' Match in SML !!!
        
        //the user hits linear univariate equation on the hierarchy
        //this _NOT_ a matching problem:
        id = new ProblemID();
        id.addString("linear");
        id.addString("univariate");
        id.addString("equation");
        m = ci.tryMatchProblem(id);
        assertEquals(
        m.getModel().getWhere().toString(),
        "[\"false (False)\", "//WN050626 this seems to be for
        // SMLtesting
        + "\"correct (~ lhs (x ^ 2 + 4 * x + 5 = 2) is_polyrat_in x)\", "
        + "\"correct (~ rhs (x ^ 2 + 4 * x + 5 = 2) is_polyrat_in x)\", "
        + "\"false (lhs (x ^ 2 + 4 * x + 5 = 2) has_degree_in x = 1)\", "
        + "\"false (rhs (x ^ 2 + 4 * x + 5 = 2) has_degree_in x = 1)\"]");
        //end step_f
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
        //search for a matching problem in the problem-hierarchy:
        //this search _FAILES_ because refine does _NOT_ return a new ProblemID
        m = ci.tryRefineProblem(id);
        assertEquals(m.getKEStoreID().toSMLString(),
        "[\"linear\",\"univariate\",\"equation\"]");
        //this search is _SUCCESSFUL_ because it returns a _NEW_ ProblemID
        //(and in this case the Model will _ALWAYS_ be "correct"!)
        id = new ProblemID();
        id.addString("univariate");
        id.addString("equation");
        m = ci.tryRefineProblem(id);
        assertEquals(m.getKEStoreID().toSMLString(),
        "[\"normalize\",\"polynomial\",\"univariate\",\"equation\"]");
        //end step_g
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
        //transfer the problem found back to the Worksheet
        Vector<String> keylist = new Vector<>();
        keylist.add("normalize");
        keylist.add("polynomial");
        keylist.add("univariate");
        keylist.add("equation");
        ch = ct.setProblem(new ProblemID(keylist));
        System.out.println(".ch=" + ch.toSMLString());
        //FIXXME.SR.WN050426 in the CalcHead above the Problem should be set
        // !!!
        //and the calchead_status_ should be still "incomplete" due to
        // ..incomplete specification (theory, method missing)
        //after inserting Theory and Method the CalcHead is complete:
        ch = ct.nextSpecify();
        ch = ct.nextSpecify();
        assertEquals(ch.getStatusString(), "correct");
        
        System.out.println("---END isac.bridge.TestSpecify#testMatchRefine");*/
    }

    /**
     * @see \label{Testcase???}, functional test ???
     */
    public void testFillModelByHand() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestSpecify#testFillModelByHand(");

        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model ...
        // could be continued by nextSpecify(), i.e. by <Next>-button,
        // but here by stepwise input
        ch = (CalcHead) ci.getElement();
        System.out.println(ch.toSMLString());
        ModelItem item1 = new ModelItem();
        item1.setText("fixedValues [r = Arbfix]");
        ModelItemList given_list = new ModelItemList();
        given_list.addItem(item1);
        ch.getModel().setGiven(given_list);
        System.out.println(ch.toSMLString());

        // TODO REPLACE WITH initContext
        //Match m = ci.initMatchProblem();

        // TODO REPLACE WITH checkContext
        ProblemID id = new ProblemID();
        id.addString("maximum_of");
        id.addString("function");

        //m = ci.tryMatchProblem(id);
        //WN051125 not checked, may contain helpful code ???...
        //        System.out.println(m.getModel().getGiven());
        //
        //        assertEquals(ch.getSpecification().toSMLString(),
        //                "\"e_domID\",[\"e_pblID\"],[\"e_metID\"]");
        //        //input the Theory ...
        //        Tactic tac = new SimpleTactic("Specify_Theory", "DiffApp.thy");
        //        ct.setNextTactic(tac);
        //        ch = (CalcHead) ci.getElement();
        //        assertEquals(ch.getSpecification().toSMLString(),
        //                "\"DiffApp.thy\",[\"e_pblID\"],[\"e_metID\"]");
        //        //input the Problem ...
        //        Vector<String> keylist = new Vector<>();
        //        keylist.add("maximum_of");
        //        keylist.add("function");
        //        tac = new StringListTactic("Specify_Problem", keylist);
        //        ct.setNextTactic(tac);
        //        //input the Method ...
        //        keylist = new Vector<>();
        //        keylist.add("DiffApp");
        //        keylist.add("max_by_calculus");
        //        tac = new StringListTactic("Specify_Method", keylist);
        //        ct.setNextTactic(tac);
        //        ch = (CalcHead) ci.getElement();
        //        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
        //                + "[\"maximum_of\",\"function\"],"
        //                + "[\"DiffApp\",\"max_by_calculus\"]");
        //        System.out
        //                .println("---END isac.bridge.TestSpecify#testFillModelByHand");
    }

    /**
     * @see \label{Testcase???}, functional test ???
     */
    public void testSwitchModelGuard() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestSpecify#testSwitchModelGuard(");
        //start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToDialog();
        ct.addDataChangeListener(user);
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        ch = ct.completeModel();

        //get the problems Model (default)-----------------------------------
        String test = "\nModel:\n"//.........................................
                + "Given:\n"//.........................................
                + "[\"correct (fixedValues [r = Arbfix])\"]\n"//.........
                + "Where:\n"//.........................................
                + "[]\n"//.........................................
                + "Find:\n"//.........................................
                + "[\"correct (maximum A)\", \"correct (valuesFor [a, b])\"]\n"
                + "Relate:\n"//.........................................
                + "[\"correct (relations [A = a * b, "
                + "(a / 2) ^ 2 + (b / 2) ^ 2 = r ^ 2])\"]";
        ch = (CalcHead) ci.getElement();
        assertEquals("the pbl's Model: ", ch.getModel().toString(), test);

        //get the methods guard ----------------------------------------------
        ci.switchToGuard();
        ch = (CalcHead) ci.getElement();
        System.out.println("the met's guard: " + ch.getModel().toString());
        //FIXXXME.WN050811 wrong: still returns pbl's Model

        //switch back to the Model -------------------------------------------
        ci.switchToModel();
        System.out.println("---END isac.bridge.TestSpecify#testFillModelByHand");
    }
}