/*
 * @author Richard Gradischnegg RG
 * Created on Oct 20, 2003
 */

package isac.bridge;

import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Context;
import isac.util.formulae.ContextMethod;
import isac.util.formulae.ContextProblem;
import isac.util.formulae.ContextTheory;
import isac.util.formulae.Match;
import isac.util.formulae.MethodID;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.tactics.Tactic;
import isac.wsdialog.IContextProvider.ContextType;

import java.io.Serializable;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * Class MathEngine: This class is called by the Dialog and communicates with
 * the Bridge via RMI. It also delegates the methods of the CalcTree and the
 * CalcIterator to the Bridge.
 * 
 * @author Richard Gradischnegg RG
 */
public class MathEngine implements Serializable {

    static final long serialVersionUID = 500L;

    private static final Logger logger = Logger.getLogger(MathEngine.class
            .getName());

    private static MathEngine singleton_;

    private IBridgeRMI bridge_rmi_;

    // Stores the calc_trees_ after call of getCalcTree
    private Map<Integer, Object> calc_trees_;

    // This is a Singleton Class: A private constructor is needed
    private MathEngine(String hostName) {
        if (logger.isDebugEnabled())
            logger.debug("MathEngine(" + hostName + ")");
        try {
            // connect to bridge
            bridge_rmi_ = (IBridgeRMI) Naming.lookup("//" + hostName
                    + "/BridgeRMI");
            System.out.println("MathEngine Constr: connected to Bridge: "
                    + bridge_rmi_.toString());
            if (bridge_rmi_ == null)
                System.out
                        .println("Error occured: The Bridge could not be found!");
        } catch (Exception e) {
            System.out
                    .println("Could not connect to Bridge via RMI\n"
                            + "Please make sure that the Bridge is running and connected");
            e.printStackTrace();
        }
        calc_trees_ = new HashMap<Integer, Object>();
    }

    /*
     * FIXME.WN040924 just to make the old bridge continue work
     */
    protected IBridgeRMI getBridgeRMI() {
        return bridge_rmi_;
    }

    /**
     * Initialize the MathEngine
     * 
     * @param hostName:
     *            Host name on which the Bridge is running
     */
    public static void init(String hostName) {
        if (logger.isInfoEnabled())
            logger.info("  DG->BR: init(" + hostName + ")");
        if (singleton_ == null) {
            singleton_ = new MathEngine(hostName);
        }
    }

    public static MathEngine getMathEngine() {
        if (logger.isInfoEnabled())
            logger.info("  DG<>BR: getMathEngine()");
        return singleton_;
    }

    /**
     * Start a new calculation
     * 
     * @param f
     *            Formalization of the new calculation, from expl
     * @return CalcHead: empty for startSpecifying
     * @throws RemoteException
     *             WN040924 ???
     * 
     * int id = calc_head_.getCalcTreeID();//WN040922 TODO simplify CalcHead
     * CalcTree calcTree = (CalcTree) calc_trees_.get(new
     * Integer(id));//WN040922 TODO simplify
     */
    public CalcTree getCalcTree(Formalization f) {

        String fmzSMLString = null;

        if (f != null)
            fmzSMLString = f.toSMLString();

        if (logger.isInfoEnabled())
            logger.info("  DG->BR: getCalcTree(" + fmzSMLString + ")");

        CalcTree calcTree = null;// WN
        try {
//            System.out.println(bridge_rmi_);
//            System.out.println(f);
            int id = bridge_rmi_.getCalcTree(f);
            calcTree = new CalcTree(this, id);
            calc_trees_.put(new Integer(id), calcTree);

            ICalcIterator hotSpot = calcTree.iterator();
            calcTree.setHotSpot(hotSpot); // first iterator marks the hotSpot
            // hotSpot.moveRoot();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        if (logger.isInfoEnabled())
            logger.info("  DG<-BR: getCalcTree <- calcTree=" + calcTree);
        return calcTree;
    }

    // Delegated methods from CalcTree/Iterator
    CChanged startCalculate(int id) throws RemoteException {
        return bridge_rmi_.startCalculate(id);
    }

    // for specify phase; these methods return a CalcHead,
    // and dont throw a CalcChanged like autoCalculate etc.

    /**
     * @see isac.interfaces.IToCalc#modelProblem()
     */
    public CalcHead modelProblem(int calc_tree_id) throws RemoteException {
        return bridge_rmi_.modelProblem(calc_tree_id);
    }

    /**
     * @see isac.interfaces.IToCalc#nextSpecify()
     */
    public CalcHead nextSpecify(int calc_tree_id) throws RemoteException {
        return bridge_rmi_.nextSpecify(calc_tree_id);
    }

    /**
     * @see isac.interfaces.IToCalc#checkCalcHead()
     */
    public CalcHead checkCalcHead(int calc_tree_id, CalcHead calchead)
            throws RemoteException {
        return bridge_rmi_.checkCalcHead(calc_tree_id, calchead);
    }

    /**
     * @see isac.interfaces.IToCalc#completeModel()
     */
    public CalcHead completeModel(int calc_tree_id) throws RemoteException {
        return bridge_rmi_.completeModel(calc_tree_id);
    }

    /**
     * @param thy_id
     * @see isac.interfaces.IToCalc#setTheory()
     */
    public CalcHead setTheory(int calc_tree_id, String thy_id)
            throws RemoteException {
        return bridge_rmi_.setTheory(calc_tree_id, thy_id);
    }

    /**
     * @see isac.interfaces.IToCalc#setProblem()
     */
    public CalcHead setProblem(int calc_tree_id, ProblemID id)
            throws RemoteException {
        return bridge_rmi_.setProblem(calc_tree_id, id);
    }

    /**
     * @see isac.interfaces.IToCalc#setMethod()
     */
    public CalcHead setMethod(int calc_tree_id, MethodID id)
            throws RemoteException {
        return bridge_rmi_.setMethod(calc_tree_id, id);
    }

    /**
     * @see isac.interfaces.IToCalc#completeCalcHead()
     */
    public CalcHead completeCalcHead(int calc_tree_id) throws RemoteException {
        return bridge_rmi_.completeCalcHead(calc_tree_id);
    }

    /**
     * @see isac.interfaces.IToCalc#completeCalcHead()
     * @deprecated because not yet implemented in SML
     */
    public CalcHead completeCalcHead(int calc_tree_id, CalcHead calcHead,
            int completeItem) throws RemoteException {
        return null;
    }

    /**
     * @see isac.interfaces.IToCalc#resetCalcHead()
     */
    public CalcHead resetCalcHead(int calc_tree_id) throws RemoteException {
        return bridge_rmi_.resetCalcHead(calc_tree_id);
    }

    int setNextTactic(int id, Tactic tactic) throws RemoteException {
        return bridge_rmi_.setNextTactic(id, tactic);
    }

    Tactic fetchProposedTactic(int id) throws RemoteException {
        return bridge_rmi_.fetchProposedTactic(id);
    }

    CEvent autoCalculate(int id, int scope, int nSteps)
            throws RemoteException {
        if (logger.isDebugEnabled())
            logger.debug("autoCalculate: id=" + id + ", scope=" + scope
                    + ", nSteps=" + nSteps);
        return bridge_rmi_.autoCalculate(id, scope, nSteps);
    }

    CEvent replaceFormula(int id, CalcFormula f) throws RemoteException {
        return bridge_rmi_.replaceFormula(id, f);
    }

    CEvent appendFormula(int id, CalcFormula f) throws RemoteException {
        return bridge_rmi_.appendFormula(id, f);
    }

    CEvent intermediateSteps(int id, ICalcIterator ci) 
            throws RemoteException {
        return bridge_rmi_.intermediateSteps(id, ci);
    }

    Vector getFormulaeFromTo(int id, ICalcIterator iterator_from,
            ICalcIterator iterator_to, Integer level,
            boolean result_includes_tactics) throws RemoteException {
        return bridge_rmi_.getFormulaeFromTo(id, iterator_from, iterator_to,
                level, result_includes_tactics);
    }

    Position moveActiveFormula(int calcTreeID, Position p)
            throws RemoteException {
        return bridge_rmi_.moveActiveFormula(calcTreeID, p);
    }

    int iterator(int id) throws RemoteException {
        if (logger.isDebugEnabled())
            logger.debug("iterator: id=" + id);
        return bridge_rmi_.iterator(id);
    }

    Position moveRoot(int calcTreeID, int iteratorID) throws RemoteException {
        if (logger.isDebugEnabled())
            logger.debug("moveRoot: calcTreeID=" + calcTreeID + ", iteratorID="
                    + iteratorID);
        return bridge_rmi_.moveRoot(calcTreeID, iteratorID);
    }

    Position moveUp(int calcTreeID, int iteratorID, Position p)
            throws RemoteException {
        return bridge_rmi_.moveUp(calcTreeID, iteratorID, p);
    }

    Position moveDown(int calcTreeID, int iteratorID, Position p)
            throws RemoteException {
        if (logger.isDebugEnabled())
            logger.debug("moveDown: calcTreeID=" + calcTreeID + ", sml_pos_="
                    + p);
        return bridge_rmi_.moveDown(calcTreeID, iteratorID, p);
    }

    Position moveLevelUp(int calcTreeID, int iteratorID, Position p)
            throws RemoteException {
        return bridge_rmi_.moveLevelUp(calcTreeID, iteratorID, p);
    }

    Position moveLevelDown(int calcTreeID, int iteratorID, Position p)
            throws RemoteException {
        return bridge_rmi_.moveLevelDown(calcTreeID, iteratorID, p);
    }

    Position moveCalcHead(int calcTreeID, int iteratorID, Position p)
            throws RemoteException {
        return bridge_rmi_.moveCalcHead(calcTreeID, iteratorID, p);
    }

    ICalcElement getElement(int calcTreeID, Position p) throws RemoteException {
        return bridge_rmi_.getElement(calcTreeID, p);
    }

    /**
     * Destruct a calcTree: Gives the memory occupied by this calcTree free
     * again. Call this method when the calcTree is no longer needed. This
     * method is irreversible.
     * 
     * @param calcTreeID
     *            id of the calcTree to be destructed
     */
    public boolean destruct(int calcTreeID) throws RemoteException {
        return bridge_rmi_.destruct(calcTreeID);
    }

    /**
     * Save a calcTree
     * 
     * @param calcTreeID
     *            id of the calcTree to be saved
     * @return a vector containing strings, representing the status of the
     *         calcTree
     */
    public Vector saveCalcTree(int calcTreeID) {
        try {
            return bridge_rmi_.saveCalcTree(calcTreeID);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Load a previously stored calcTree
     * 
     * @param v
     *            Vector: the return value of saveCalcTree
     * @return a calcTree reference
     */
    public CalcTree loadCalcTree(Vector v) {
        try {
            int id = bridge_rmi_.loadCalcTree(v);
            CalcTree calcTree = new CalcTree(this, id);
            calc_trees_.put(new Integer(id), calcTree);
            ICalcIterator hotSpot = calcTree.iterator();
            // may be not first iterator (not hotspot)
            ((CalcIterator) hotSpot).makeHotSpot();
            // make sure it is a hotspot iterator
            calcTree.setHotSpot(hotSpot);
            hotSpot.moveRoot();
            return calcTree;
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * @see isac.interfaces.ICalcIterator#getTactic()
     */
    public Tactic getTactic(int id, Position pos) throws RemoteException {
        return bridge_rmi_.getTactic(id, pos);
    }

    public Vector getApplicableTactics(int calcTreeID, int scope, Position pos)
            throws RemoteException {
        return bridge_rmi_.getAppliableTactics(calcTreeID, scope, pos);
    }

    /**
     * @see isac.interfaces.ICalcIterator#getAssumptions()
     */
    public Assumptions getAssumptions(int id, Position pos)
            throws RemoteException {
        return bridge_rmi_.getAssumptions(id, pos);
    }

    /**
     * @see isac.interfaces.ICalcIterator#getAssumptions()
     */
    public Assumptions getAccumulatedAssumptions(int id, Position pos)
            throws RemoteException {
        return bridge_rmi_.getAccumulatedAssumptions(id, pos);
    }

    /**
     * @see isac.interfaces.ICalcIterator#initMatchProblem
     */
    public Match initMatchProblem(int calcTreeID) throws RemoteException {
        return bridge_rmi_.initMatchProblem(calcTreeID);
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryMatchProblem
     */
    public Match tryMatchProblem(int calcTreeID, ProblemID pblID)
            throws RemoteException {
        return bridge_rmi_.tryMatchProblem(calcTreeID, pblID);
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryRefineProblem
     */
    public Match tryRefineProblem(int calcTreeID, ProblemID pblID)
            throws RemoteException {
        return bridge_rmi_.tryRefineProblem(calcTreeID, pblID);
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryMatchMethod
     */
    public Match initMatchMethod(int calcTreeID) throws RemoteException {
        return bridge_rmi_.initMatchMethod(calcTreeID);
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryMatchMethod
     */
    public Match tryMatchMethod(int calcTreeID, MethodID pblID)
            throws RemoteException {
        return bridge_rmi_.tryMatchMethod(calcTreeID, pblID);
    }

    public Context initContext(int calcTreeID, ContextType type, Position pos)
            throws RemoteException {
        return bridge_rmi_.initContext(calcTreeID, type, pos);
    }

    public Context checkContext(int javaCalcID, Context context, Position pos)
            throws RemoteException {
        return bridge_rmi_.checkContext(javaCalcID, context, pos);
    }

    public Context refineProblem(int javaCalcID, Context context, Position pos)
            throws RemoteException {
        return bridge_rmi_.refineProblem(javaCalcID, context, pos);
    }

    public CalcHead setContext(int javaCalcID, ContextProblem context,
            Position pos) throws RemoteException {
        return bridge_rmi_.setContext(javaCalcID, context, pos);
    }

    public CalcHead setContext(int javaCalcID, ContextMethod context,
            Position pos) throws RemoteException {
        return bridge_rmi_.setContext(javaCalcID, context, pos);
    }

    public CChanged setContext(int javaCalcID, ContextTheory context,
            Position pos) throws RemoteException {
        return bridge_rmi_.setContext(javaCalcID, context, pos);
    }
}