/*
 * @author Georg Kompacher, member of the ISAC-team, 
 * Copyright (c) 2006 by Georg Kompacher
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.browserdialog;

import isac.session.Session;
import isac.session.ILMService.SessionState;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UserActionOnLink;
import isac.util.Formalization;
import isac.interfaces.IToGUI;
import isac.session.DialogGuide;
import isac.util.formulae.Context;
import isac.util.formulae.KEStoreKey;
import isac.util.parser.FormalizationDigest;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IContextProvider.ContextType;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.Vector;

/**
 * @author Georg Kompacher
 * 
 * This class is responsible for the interactions to the ExampleBrowser, to the
 * active WSDialog, and also to other BrowserDialogs.
 */
public class ExampleDialog extends BrowserDialog {

    /**
     * For serialization, see corresponding API Documentation; value generated
     * by eclipse to avoid a warning
     */
    private static final long serialVersionUID = 338252357001819521L;
    
    public ExampleDialog(Session session, DialogGuide dialog) throws RemoteException {
        super(session, dialog);
        context_type_ = ContextType.CONTEXT_EXAMPLES;

        hierarchy_ = loadHierarchy(context_type_);
    }

    /**
     * opens a worksheet from an example specified by the filename. The xml
     * content of the file is loaded and a new Formalisation is created with
     * this xml string. A new worksheet is opened by use of the WSDialogManager.
     * 
     * @param filename
     *            the name of the xml file of the example to start
     * @return true if everything went ok, and false otherwise
     */
    public boolean openWorksheetFromExample(String filename) {
        try {
            System.out.println("filename open: " + filename);
            Vector<String> xml_result = loadContent(session_.getId(), "exp", filename);
            String xml_string = (String) xml_result.get(0);

            FormalizationDigest fd = new FormalizationDigest();
            Formalization formalization = fd.parseFormalization(xml_string);
            // FIXXME: RK060725 in future there will be no .xml
            KEStoreKey example_id = new KEStoreKey(filename.replaceAll(".xml",
                    ""));

            if (formalization == null) {
                System.out.println("Error: Could not find formalization for example '" + example_id + "'");
            } else {
                session_.getWSDialogManager().openWSDialog(formalization,
                        example_id);
            }
        } catch (RemoteException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    /**
     * Notify the ExampleDialog about events or requests triggered by the user.
     * The ExampleBrowser calls this method over RMI.
     * 
     * @param action
     *            What's new from the UI
     * @return true, if the event is being processed, false if processing is
     *         denied or impossible
     * @throws RemoteException
     * @throws DialogProtocolException
     */
    public boolean notifyUserAction(IUserAction action)
            throws DialogProtocolException, RemoteException {
//        System.out.println("ExampleDialog::notifyUserAction()"
//                + action.getActionID());

        if (super.notifyUserAction(action))
            return true;

//UL         user_logger_.logUserAction("exp", action);

        switch (action.getActionID()) {
        case UI_MINIBROWSER_LINK: {
            URL receivedLink = ((UserActionOnLink) action).getLink();
            // ATTENTION: if link has another browser as destination, the path
            // is still to the source-browser (Design-Error?)
            try {
                interpretLink(receivedLink);
                return true;
            } catch (RemoteException e) {
                e.printStackTrace();
                return false;
            }
        }
        case UI_BROWSER_MATCH_ON: {
            context_on_ = true;
            drawButtons();
            sendInitContextToBrowser();
            return true;
        }
        case UI_BROWSER_MATCH_OFF: {
            context_on_ = false;
            drawButtons();
            return true;
        }
        default: break;
        }
        return false;
    }

    public void registerBrowserFrame(IToGUI browser_frame)
            throws RemoteException {
        super.registerBrowserFrame(browser_frame);
        initButtons();
    }

    private void initButtons() {
        context_on_visible_ = false;
        context_on_ = true;
        try {
            browser_frame_rmi_.addUIElement(new UIAction(
                    EUIElement.UI_BROWSER_MATCH_OFF,
                    EUIContext.UI_CONTEXT_BROWSER, user_language_));
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    protected void drawButtons() {
        if (!context_on_) {
            drawButtonContextOn();
        } else {
            drawButtonContextOff();
        }
    }

    /**
     * This method checks the received context by use of the checkContext method
     * of the IContextProvider interface. This interface returns again a
     * context, which has to be displayed.
     * 
     * @param context
     *            the received context which has to be checked.
     */
    public void sendCheckedContextToBrowser(Context context) {
        sendContextToBrowser(context);
        sendLinkToMiniBrowser(context, context_type_, "");
    }

    /**
     * triggers the ExampleBrowser to ask Compod for a first example
     * 
     * @deprecated WN made Compod run quick and dirty
     * 
     * better involve the UserModel !!!
     */
    public void notifyStartFirstExample() {
        String user_name = super.session_.getUser().getUsername();
        String[] next_objects;
        try {
            if (SessionState.ACTIVE == super.getCompod().getSessionState(
                    user_name)) {
                next_objects = super.getCompod().getNextObjects(user_name);
                // always take the first object
                openWorksheetFromExample(next_objects[0]);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * triggers the ExampleBrowser to ask Compod for an example to be started
     * next.
     * 
     * @param exp_id
     *            the unique identifier in the database of examples for the
     *            example just finished; required for Compod (and later on for
     *            UserLogger)
     * @param success_info
     *            0.0 | 1.0 (TODO.WN090111 isac.users.successInfo instead)
     * 
     * @deprecated WN08 made Compod run quick and dirty
     * 
     * better involve the UserModel !!!
     */
    public void notifySomeExampleFinished(String exp_id, float success_info) {
        String user_name = super.session_.getUser().getUsername();
        String[] next_objects;
        try {
            if (success_info == 0.0)
                super.getCompod().notifyObjectResult(user_name,
                        exp_id + ".xml", false);
            else
                super.getCompod().notifyObjectResult(user_name,
                        exp_id + ".xml", true);
            if (SessionState.ACTIVE == super.getCompod().getSessionState(
                    user_name)) {
                next_objects = super.getCompod().getNextObjects(user_name);
                // always take the first object
                openWorksheetFromExample(next_objects[0]);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}