/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.gui;

import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnBrowserDialog;
import isac.useractions.UIActionOnWSDialog;
import isac.useractions.UserAction;
import isac.users.IUserManager;
import isac.util.FixedPortRMISocketFactory;
import isac.util.WindowApplicationPaths;
import isac.gui.browser.BrowserFrameSwing;
import isac.gui.browser.BrowserFrameRMI;
import isac.interfaces.IToGUI;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IWorksheetDialog;
import isac.browserdialog.IBrowserDialog;
import isac.session.IObjectManager;
import isac.session.ISession;

import isac.session.LearningMode;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.RMISocketFactory;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;
import java.util.Properties;

import javax.swing.*;
import javax.swing.plaf.basic.BasicInternalFrameUI;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.Logger;

/**
 * Main window application for the ISAC system. Responsible for showing the user
 * all requested information and for the interaction with the user.
 * 
 * @author Mario Hochreiter
 * @author Manuel Koschuch
 * 
 */
public class WindowApplication extends UnicastRemoteObject implements
    ActionListener, ILoginListener, IToWorksheetUser, IGuiConstants, IToGUI, Serializable {

    private static final long serialVersionUID = 78965443;
    
    private static class CommandLineOptions {

        public static final String HELP = "help";
        public static final String LEARNING_MODE = "mode";
        public static final String PASSWORD = "password";
        public static final String EXAMPLE = "example";
        public static final String USERNAME = "username";
    }
    
    protected static class CommandLineValues {

        public String learningMode = null;
        public String username = null;
        public String password = null;
        public String example = null;

        public LearningMode getValidLearningMode() {
            return LearningMode.getLearningModeOrDefault(learningMode, LearningMode.LEARNER);
        }

        public boolean hasValidLoginCredentials() {
            return username != null && password != null;
        }
    }
    private CommandLineValues commandLineValues = new CommandLineValues();
    private static final Logger LOG = Logger.getLogger(WindowApplication.class.getName());
    private static JFrame mainFrame;
    /**
     * Handles the user-action clicking X to close WindowApplication. WN071011
     * copied from http://mindprod.com/jgloss/close.html
     */
    private static WindowListener windowListener = new WindowAdapter() {

        public void windowClosing(WindowEvent w) {
            try {
                currentSession.notifyUserAction(new UserAction(
                    EUIElement.UI_STOP_SESSION,
                    EUIContext.UI_CONTEXT_WINDOW_APPLICATION));
            } catch (Exception e) {
                defaultHandleRemoteException(e);
            } finally {
                mainFrame.setVisible(false);
                mainFrame.dispose();
                // WN071011 otherwise does not cancel WA in the VM
                System.exit(0);
            }
        }
    };
    private IUserManager userManager;
    private static ISession currentSession;
    /** Container for the internal frames, which will contain the worksheets */
    private JDesktopPane worksheetDesktopPane;
    /** login dialogue */
    private LoginScreen loginScreen;
    // hash map containing the dynamical created buttons
    protected HashMap<JButton, EUIElement> map_buttons_actions_;
    protected HashMap<EUIElement, JButton> map_actions_buttons_;
    private JMenuBar menu_bar_;
    private boolean menu_separator_added_;
    private int num_browser_frames_;

    public WindowApplication() throws RemoteException {

        menu_separator_added_ = false;

        mainFrame = new JFrame();
        mainFrame.setSize(new Dimension(WINDOW_APPLICATION_INITIAL_SIZE_X,
            WINDOW_APPLICATION_INITIAL_SIZE_Y));
        Point screenCenter = this.screenCenter();
        mainFrame.setLocation(screenCenter);

        // initialize hashMaps
        map_buttons_actions_ = new HashMap<JButton, EUIElement>();
        map_actions_buttons_ = new HashMap<EUIElement, JButton>();

        menu_bar_ = new JMenuBar();

        mainFrame.setJMenuBar(menu_bar_);

        num_browser_frames_ = 0;

        this.createWorksheetPane();

        mainFrame.add(worksheetDesktopPane);

        JFrame.setDefaultLookAndFeelDecorated(true);

        // WN071011 handle 'close WindowApplication' with windowListener
        // main_frame_.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        mainFrame.setVisible(false);
    }

    private void createLoginScreen() {
        loginScreen = new LoginScreen();
        loginScreen.addListener(this);
        loginScreen.setVisible(false);
    }

    /* LK 04.08.27 */
    /**
     * login method called from <code>LoginScreen</code> Initialize RMI
     * connection to <code>ObjectManager</code> and
     * <code>XSessionManager</code>.
     */
    private void login(String username, String password, LearningMode learningMode, String example) {
        String name = WindowApplicationPaths.OBJECT_MANAGER_RMI;

        IObjectManager objMan = null;
        try {
            System.out.println("connect to ObjectManager in getSDialog : "
                    + name);
            objMan = (IObjectManager) Naming.lookup(name);
            
            userManager = objMan.getUserManager();
        } catch (RemoteException exc) {
            System.err.println("no rmiregistry runing on server " + name
                + " no connection possible --- try later");
            System.err.println("serverCom exception: " + exc.getMessage());
            exc.printStackTrace();
        } catch (NotBoundException exc) {
            System.err.println("servercom: notboundexc: " + exc.getMessage());
            System.err.println("server not found - try to connect to Proxy");
        } catch (java.net.MalformedURLException exc) {
            System.err.println("servercom: MalformedURLException: "
                    + exc.getMessage());
            exc.printStackTrace();
        }
        
        if (LOG.isInfoEnabled()) {
            LOG.info("WA->UM: login");
        }
        try {
            currentSession = userManager.login(username, password, learningMode);
            if (currentSession == null) {
                loginScreen.setVisible(true);
                loginScreen.showErrorMsgWrongUsernameOrPassword(username, password);
                System.out.println("Could not create session, probably username '"
                    + username + "' or password '" + password + "' is wrong");
            } else {
                System.out.println("Login success, starting 'current_session_."
                    + "registerBrowserFrame((IToGUI) this);'\n"
                    + "?Error?: if the system stops here some minutes, this is because the ObjectManager cannot connect"
                    + "back to the WindowApplication via javaRMI\n"
                    + "Solution: WindowApplication and ObjectManager MUST RUN WITHIN THE SAME SUB-NET !");
                currentSession.registerBrowserFrame((IToGUI) this);
                if (example == null || !example.trim().isEmpty()) {
                    objMan.getSessionDialog().getFirstSession(username).openWorksheetFromExample(example);
                }
                System.out.println("NO Error: registerBrowserFrame success. New session with user: " + username);
            }
        } catch (RemoteException e) {
            loginScreen.showErrorMsg();
            System.err.println("PANIC RK isac.gui.Windowapplication#login(String,LearningMode,String)");
            System.err.println("login exception: " + e.getMessage());
            System.err.println("Error detection: compare the IPs of the error above "
                + "with the IPs of WindowApplication and ObjectManager.");
            if (LOG.isDebugEnabled()) {
                LOG.error("login", e.detail);
            }
        } finally {
            loginScreen.dispose();
            mainFrame.setVisible(true);
        }
    }

    /**
     * Logs a user in via command line or via login screen.
     */
    void startLogin() {
        createLoginScreen();
        if (commandLineValues.hasValidLoginCredentials()) {
            login(commandLineValues.username, commandLineValues.password,
                commandLineValues.getValidLearningMode(), commandLineValues.example);
        } else {
            loginScreen.setVisible(true);
        }
    }

    /**
     * To add dynamic buttons to the menu of the frame.
     * 
     * @param action
     *            The action assigned to the button.
     */
    public void addToMenuBar(IUIAction action) {
        EUIElement action_id = action.getAction();

        // / TODO: Really bad HACK ... This should be rewritten ...
        // / One possible sollution is set the layout of the main frame to
        // / BorderLayout, add a splitpane to the north part of the main_frame_
        // / and add buttons that have UI_CONTEXT_WINDOW_APPLICATION to the
        // panel
        // / in the left part of the splitpane and the buttons that have
        // / UI_CONTEXT_CALCULATION to the panel in the right part of the
        // splitpane ...
        if (!menu_separator_added_
            && action.getContext() == EUIContext.UI_CONTEXT_CALCULATION) {
            menu_bar_.add(new JLabel("             "));
            menu_bar_.add(new JLabel("             "));
            menu_bar_.add(new JLabel("             "));
            menu_bar_.add(new JLabel("             "));
            menu_separator_added_ = true;
        }

        // check if a button is already in the menu
        if (!map_actions_buttons_.containsKey(action_id)) {
            JButton dynamic_button = new JButton(action.getLabel());
            dynamic_button.setName(action.getLabel());
            dynamic_button.addActionListener(this);

            map_actions_buttons_.put(action_id, dynamic_button);
            map_buttons_actions_.put(dynamic_button, action_id);

            menu_bar_.add(dynamic_button);
            menu_bar_.updateUI();
        }
    }

    /**
     * Removes item from menu.
     * 
     * @param action
     * @return true if done else false
     */
    public boolean removeFromMenuBar(IUIAction action) {

        // / get the ID of the action
        EUIElement actionID = action.getAction();

        // / check if the button is registered
        if (!map_actions_buttons_.containsKey(actionID)) {
            System.out.println("In BrowserFrame::removeFromMenuBar() - The button is not registered !!");
            return false;
        } else {
            JButton button_to_delete = (JButton) map_actions_buttons_.get(actionID);

            map_actions_buttons_.remove(actionID);
            map_buttons_actions_.remove(button_to_delete);

            menu_bar_.remove(button_to_delete);
            menu_bar_.updateUI();

            return true;
        }
    }

    /**
     * Open a new Worksheet and start the modeling and specification phase.
     * 
     * @param fmz
     *            Formalization (can be empty see UC 2.5.4) that the math engine
     *            needs to start a calculation
     * @param source
     *            Source of the Formalization (e.g. ExampleHierarchy or from
     *            Scratch)
     * @param type
     *            Requested ViewType for the CalcHead
     * @throws RemoteException
     */
    private void openNewWorksheet(IWorksheetDialog dialog)
        throws RemoteException {

        // InternalFrame, holding the Frames for the Worksheet and the
        // corresponding CalcHeadPanel
        JInternalFrame internalFrame = createInternalFrame("Worksheet", true,
            true, false, false);
        internalFrame.setResizable(true);
        internalFrame.setLayout(new BorderLayout());

        // DesktopPane, added to internalFrame to be able to hae multiple
        // InternalFrames in this InternalFrame
        final JDesktopPane desktop = new JDesktopPane();
        desktop.setSize(WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X,
            WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_Y);

        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

        internalFrame.setContentPane(desktop); // , BorderLayout.CENTER);

        mainFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        Worksheet ws = new Worksheet(this, dialog, desktop);
        // new InternalFrame for the Worksheet
        JInternalFrame ws_frame = new JInternalFrame();
        ((BasicInternalFrameUI) ws_frame.getUI()).setNorthPane(null);
        ws_frame.setSize(WORKSHEET_FRAME_INITIAL_SIZE_X, mainFrame.getHeight()
            - WORKSHEET_FRAME_INITIAL_OFFSET_Y);

        Container c;
        c = ws_frame.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(ws, BorderLayout.CENTER);

        // NC060726: The Worksheet desktop manager must not have the
        // BorderLayout
        // desktop.setLayout(new BorderLayout());
        // desktop.add(ws_frame, BorderLayout.CENTER);
        desktop.add(ws_frame);

        try {
            ws_frame.setMaximum(true);
        } catch (PropertyVetoException e) {
            e.printStackTrace();
        }

        ws_frame.setFocusable(false);
        ws_frame.setVisible(true);

        internalFrame.addInternalFrameListener(ws);

        int width, height = 0;
        height = internalFrame.getPreferredSize().height;
        width = internalFrame.getPreferredSize().width;
        // make Frame a little smaller than the window
        height = mainFrame.getHeight() - WORKSHEET_FRAME_INITIAL_OFFSET_Y;
        width = width < 600 ? WORKSHEET_FRAME_INITIAL_SIZE_X : width;

        JInternalFrame[] frames = worksheetDesktopPane.getAllFrames();
        int num_frames = frames.length - 4;

        this.addInternalFrame(internalFrame);

        internalFrame.setLocation(num_frames * 20, num_frames * 20);
        internalFrame.setPreferredSize(new Dimension(width, height));
        internalFrame.pack();
        internalFrame.setVisible(true);

        mainFrame.setCursor(Cursor.getDefaultCursor());
    }

    private JInternalFrame createInternalFrame(String title, boolean resizable,
        boolean closable, boolean maximizable, boolean iconifiable) {
        JInternalFrame internalFrame = new JInternalFrame(title, resizable,
            closable, maximizable, iconifiable);
        return internalFrame;
    }

    private void createWorksheetPane() {
        worksheetDesktopPane = new JDesktopPane();
        Dimension d = new Dimension();
        d.setSize(mainFrame.getWidth() * 0.60, 0.0);
        worksheetDesktopPane.setMinimumSize(d);
        // "ghost frame" dragging; very fast but weired for some user
        worksheetDesktopPane.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
    }

    private Point screenCenter() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int midX = screenSize.width / 2 - mainFrame.getWidth() / 2;
        int midY = screenSize.height / 2 - mainFrame.getHeight() / 2;
        return new Point(midX, midY);
    }

    private void addInternalFrame(JInternalFrame frame) {
        worksheetDesktopPane.add(frame);
        worksheetDesktopPane.getDesktopManager().activateFrame(frame);
        frame.toFront();
        frame.grabFocus();
    }

    // ----------ActionListener----------------------------------------
    public void actionPerformed(ActionEvent event) {
        System.out.println("In WindowApplication::actionPerformed(ActionEvent event)");

        // / get the source of the event
        Object event_source = event.getSource();

        // / check if it is a button
        if (event_source instanceof JButton) {
            JButton src_button = (JButton) event_source;

            // / check the map and get the actionID
            EUIElement actionID = map_buttons_actions_.get(src_button);

            // / if the button is registered
            if (actionID != null) {
                notifySession(actionID);
            } else {
                System.out.println("Got event from a button, but the button is not registered !!!");
            }
        } else {
            System.out.println("Got event from someone else!!!");
        }
    }

    private void notifySession(EUIElement action) {
        try {
            currentSession.notifyUserAction(new UserAction(action,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION));
        } catch (Exception e) {
            defaultHandleRemoteException(e);
        }
    }

    private static void defaultHandleRemoteException(Exception e) {
        if (e instanceof RemoteException || e instanceof DialogProtocolException) {
            e.printStackTrace();
        } else {
            throw new RuntimeException(e);
        }
    }

    /**
     * creates a new browser frame, inserts it to the internal frames and
     * registers it to the browser dialog
     */
    private void openNewBrowserFrame(IBrowserDialog browser_dialog,
        String frame_name) {
        try {
            BrowserFrameSwing browser_frame = new BrowserFrameSwing(
                browser_dialog, frame_name);
            addInternalFrame(browser_frame);

            num_browser_frames_++;

            browser_frame.setSize(IGuiConstants.BROWSER_MIN_SIZE_X,
                IGuiConstants.BROWSER_MIN_SIZE_Y);
            browser_frame.setLocation(IGuiConstants.BROWSER_POS_X
                + (num_browser_frames_ * 20), IGuiConstants.BROWSER_POS_Y
                + (num_browser_frames_ * 20));

            BrowserFrameRMI browser_framain_frame_rmi = new BrowserFrameRMI(
                browser_frame);

            browser_dialog.registerBrowserFrame(browser_framain_frame_rmi);

        } catch (Exception e) {
            defaultHandleRemoteException(e);
        }
    }

    /**
     * @see isac.gui.ILoginListener#loginEventOccured(isac.gui.LoginEvent)
     */
    public void loginEventOccured(LoginEvent loginEvent) {
        int type = loginEvent.getType();
        switch (type) {
            case (LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN):
                String username = loginEvent.getUsername();
                String password = loginEvent.getPassword();
                this.login(username, password, LearningMode.LEARNER, null);
                break;
            case (LoginEvent.LOGIN_TYPE_EXIT):
                mainFrame.dispose();
                System.exit(0);
                break;
        }
    }

    /**
     * This method is called periodically by the ObjectManager to ensure that
     * the RMI connection is still alive Added to fix a memory leak in
     * ObjectManager.
     * 
     * @return true if called over RMI and RMI connection is ok. If RMI
     *         connection is broken, the method throws RemoteException
     */
    public boolean isUIRunning() throws RemoteException {
        return true;
    }

    public void activateUIObject(int id, boolean active) throws RemoteException {
    }

    /**
     * WN070719 late comment
     * 
     * @deprecated in favor of isac.gui.WindowApplication#doUIAction(IUIAction
     *             action)
     */
    public boolean doUIAction(IUserAction action) throws RemoteException {

        return false;
    }

    /**
     * only for launching, handles UI_OPEN_WORKSHEET + UI_OPEN_BROWSER_FRAME only
     */
    public void doUIAction(IUIAction action) throws RemoteException {

        if (LOG.isInfoEnabled()) {
            LOG.info(("WA<-DG: doUIAction(IUIAction action) : Action is "
                + action.getAction() + ", Context is " + action.getContext()));
        }

        // if it is not our context go back
        if (action.getContext() != EUIContext.UI_CONTEXT_WINDOW_APPLICATION) {
            return;
        }

        switch (action.getAction()) {
            case UI_OPEN_WORKSHEET: {
                UIActionOnWSDialog uaowd = (UIActionOnWSDialog) action;
                openNewWorksheet(uaowd.getWorksheetDialog());
                break;
            }
            case UI_OPEN_BROWSER_FRAME:
                UIActionOnBrowserDialog uaobd = (UIActionOnBrowserDialog) action;
                openNewBrowserFrame(uaobd.getBrowserDialog(), uaobd.getFrameName());
                break;
            default: break;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see isac.interfaces.IToGUI#addUIElement(isac.useractions.IUIAction)
     */
    public void addUIElement(IUIAction action) throws RemoteException {
        if (action.getContext() == EUIContext.UI_CONTEXT_WINDOW_APPLICATION) {
            addToMenuBar(action);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see isac.interfaces.IToGUI#removeUIElement(isac.useractions.IUIAction)
     */
    public boolean removeUIElement(IUIAction action) throws RemoteException {
        if (action.getContext() == EUIContext.UI_CONTEXT_WINDOW_APPLICATION) {
            return removeFromMenuBar(action);
        }

        return false;
    }

    public void addCloseListener(Worksheet ws) {
        mainFrame.addWindowListener(ws);
    }

    public void removeCloseListener(Worksheet ws) {
        mainFrame.removeWindowListener(ws);
    }

    private static void exitIfRequiredPropertiesDontExist(Properties properties) {
        if ((WindowApplicationPaths.WORKSHEET_RMI = properties.getProperty("WORKSHEET_RMI")) == null) {
            System.err.println("No value for WORKSHEET_RMI found in property-file");
            System.exit(-1);
        }

        if ((WindowApplicationPaths.BROWSER_FRAME_RMI = properties.getProperty("BROWSER_FRAME_RMI")) == null) {
            System.err.println("No value for BROWSER_FRAME_RMI found in property-file");
            System.exit(-1);
        }

        if ((WindowApplicationPaths.OBJECT_MANAGER_RMI = properties.getProperty("OBJECT_MANAGER_RMI")) == null) {
            System.err.println("No value for OBJECT_MANAGER_RMI found in property-file");
            System.exit(-1);
        }

        if ((WindowApplicationPaths.WINDOW_APPLICATION_REVERSE_RMI_SERVER = properties.getProperty("WINDOW_APPLICATION_REVERSE_RMI_SERVER")) == null) {
            System.err.println("No value for WINDOW_APPLICATION_REVERSE_RMI_SERVER found in property-file");
            System.exit(-1);
        }

        try {
            if ((WindowApplicationPaths.WINDOW_APPLICATION_REVERSE_RMI_PORT = Integer.parseInt(properties.getProperty("WINDOW_APPLICATION_REVERSE_RMI_PORT"))) < 1) {
                System.err.println("No value for WINDOW_APPLICATION_REVERSE_RMI_PORT found in property-file");
                System.exit(-1);
            }
        } catch (NumberFormatException e1) {
            System.err.println("No value for WINDOW_APPLICATION_REVERSE_RMI_PORT found in property-file");
            System.exit(-1);
        }
    }

    private static Properties loadPropertiesFromFile(String file) {
        Properties properties = new Properties();
        InputStream is = null;
        URI uri = null; 
        try {
            uri = new URI(file);
        } catch (URISyntaxException ex) {
            try {
                uri = new URI("file", file, null);
            } catch (URISyntaxException ex1) {
                System.err.println("Could not identify source of path (" + file
                        + " ) its not an URL nor a local file");
            }
        }       
        if (uri != null) {
            try {
                is = uri.toURL().openStream();
            } catch (MalformedURLException ex) {
                System.err.println("Invalid URL for WindowApplication.properties file: " + file);
            } catch (IOException ex) {
                System.err.println("Cannot reach destination for URL of WindowApplication.properties: " + file);
            }
        }
        if (is == null) {
            System.err.println("No WindowApplication.properties file found! - SYSTEM EXITING");
            System.exit(-1);
        }
        try {
            properties.load(is);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return properties;
    }

    protected static void startApplication(final WindowApplication windowApplication) {
        // Start the application
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                try {
                    // Use ObjectManagerRMISocketFactory
                    RMISocketFactory.setSocketFactory(new FixedPortRMISocketFactory(
                        WindowApplicationPaths.WINDOW_APPLICATION_REVERSE_RMI_PORT));
                    // new
                    // WindowApplicationRMISocketFactory(WindowApplicationPaths.WINDOW_APPLICATION_REVERSE_RMI_SERVER,
                    // WindowApplicationPaths.WINDOW_APPLICATION_REVERSE_RMI_PORT)
                    // );
                } catch (IOException e) {
                    System.out.println("Exception: " + e.getMessage());
                    e.printStackTrace();
                }
                try {
                    windowApplication.startLogin();
                } catch (Throwable e) {
                    e.printStackTrace(System.err);
                } finally {
                    System.out.println("Created: " + windowApplication.toString());
                    mainFrame.addWindowListener(windowListener);
                    if (LOG.isDebugEnabled()) {
                        LOG.debug("WindowApplication started");
                    }
                }
            }
        });
    }

    public static void main(String[] args) throws RemoteException {
        System.out.println("Starting WindowApplication...");
        Options options = createPossibleCommandLineOptions();
        CommandLine line = parseCommandLineOptions(options, args); 
        CommandLineValues commandLineValues = processCommandLineOptions(line, options); 
        WindowApplication windowApplication = new WindowApplication();
        windowApplication.setCommandLineValues(commandLineValues);
        startApplication(windowApplication);
    }

    protected static Options createPossibleCommandLineOptions() throws IllegalArgumentException {
        Options options = new Options();
        OptionBuilder.withDescription("This help");
		OptionBuilder.
            withLongOpt(CommandLineOptions.HELP);
		options.addOption(OptionBuilder.create("h"));
		
        OptionBuilder.withDescription("The username to login, eg. 'ksmith' or 'domain\\ksmith'");
		OptionBuilder.hasArg();
		OptionBuilder.withArgName("user");
		OptionBuilder.withLongOpt(CommandLineOptions.USERNAME);
		options.addOption(OptionBuilder.create("u"));
		
        OptionBuilder.withDescription("The password");
		OptionBuilder.hasArg();
		OptionBuilder.withArgName("pass");
		OptionBuilder.withLongOpt(CommandLineOptions.PASSWORD);
		options.addOption(OptionBuilder.create("p"));
		
        OptionBuilder.withDescription("The learners mode: '" + LearningMode.ASSESSMENT
            + "' or '" + LearningMode.LEARNER + "'");
		OptionBuilder.hasArg();
		OptionBuilder.withArgName("learningMode");
		OptionBuilder.withLongOpt(CommandLineOptions.LEARNING_MODE);
		options.addOption(OptionBuilder.create("m"));
		
        OptionBuilder.withDescription("The example to automatically open");
		OptionBuilder.hasArg();
		OptionBuilder.withArgName("example");
		OptionBuilder.withLongOpt(CommandLineOptions.EXAMPLE);
		options.addOption(OptionBuilder.create("e"));
        return options;
    }

    protected static CommandLineValues processCommandLineOptions(CommandLine cmdLine, Options options) {
        if (cmdLine.hasOption(CommandLineOptions.HELP)) {
            printCommandLineHelp(options);
            System.exit(0);
        }
        if (cmdLine.getArgs().length < 1) {
            System.out.println("ERROR: Invalid number of arguments");
            printCommandLineHelp(options);
            System.exit(1);
        }
        Properties properties = loadPropertiesFromFile(cmdLine.getArgs()[0]);
        exitIfRequiredPropertiesDontExist(properties);

        CommandLineValues cmdLineValues = new CommandLineValues();
        cmdLineValues.learningMode = cmdLine.getOptionValue(CommandLineOptions.LEARNING_MODE);
        cmdLineValues.username = cmdLine.getOptionValue(CommandLineOptions.USERNAME);
        cmdLineValues.password = cmdLine.getOptionValue(CommandLineOptions.PASSWORD);
        cmdLineValues.example = cmdLine.getOptionValue(CommandLineOptions.EXAMPLE);
        return cmdLineValues;
    }

    protected static CommandLine parseCommandLineOptions(Options options, String[] args) {
        CommandLineParser parser = new PosixParser();
        CommandLine line = null;
        try {
            line = parser.parse(options, args);
        } catch (ParseException exp) {
            System.out.println("ERROR: The format of the commandline parameters is wrong.");
            printCommandLineHelp(options);
            System.exit(1);
        }
        return line;
    }

    private static void printCommandLineHelp(Options options) {
        HelpFormatter formatter = new HelpFormatter();
        formatter.setWidth(100);
        formatter.printHelp("WindowApplication <WindowApplication.properties file> "
            + "[-username <user> -password <pass>] [-mode <learningMode>] [-example <id of example>]",
            options);
    }
    
    public void setCommandLineValues(CommandLineValues commandLineValues) {
        this.commandLineValues = commandLineValues;
    }
}
