package isac.gui.browser;

import isac.browserdialog.IBrowserDialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JMenuBar;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import org.apache.log4j.Logger;

import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.UIActionOnMathContext;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnLink;
import isac.wsdialog.DialogProtocolException;

/**
 * 
 * This class represents the window of the Browser. It is the main dispatch
 * point for all Swing actions that happen in various sub-elements (MiniBrowser,
 * Buttons) as well as the main dispatch point where all UIActions land and get
 * dispatched according to thier context.
 * 
 * The class implements following interfaces
 * 
 * InternalFrameListener - For receiveing events from the Frame ActionListener -
 * Events from the Buttons in the MenuBar HyperLinkListener - Events from the
 * MiniBrowser
 * 
 * The class contains a MenuBar where the buttons are stored, a HierarchyPanel
 * where the tree structure is displayed and the MiniBrowser where the content
 * is presented. According to context, the actions are sent to the appropriate
 * members. For example: UIAction with UI_CONTEXT_MINIBROWSER context is
 * automatically sent to the MiniBrowser
 * 
 * The class implements notifyBrowserDialog() method that sends the action to
 * its parent dialog
 * 
 */
public class BrowserFrameSwing extends BrowserFrame implements
        HyperlinkListener, InternalFrameListener, ActionListener {

    static final long serialVersionUID = -1817363354068154971L;

    private static final Logger logger = Logger
            .getLogger(BrowserFrameSwing.class.getName());

    /**
     * 
     * This DoubleOrderedMap stores the buttons that are active on this
     * BrowserFrame. The key to the HashMap is the ID of the UserAction (the ID
     * is EUIElement) and the value to the HashMap is the actual button
     * (JButton).
     * 
     * This commons collection gives us the possibility to efficiently search
     * the HashMap both ways, with the key as well as the value.
     * 
     * Object get(Object key) Returns the value to which this map maps the
     * specified key.
     * 
     * Object getKeyForValue(Object value) Returns the key to which this map
     * maps the specified value.
     * 
     */
    protected HashMap<JButton, EUIElement> map_buttons_actions_;

    protected HashMap<EUIElement, JButton> map_actions_buttons_;

    /**
     * 
     * The menu bar containing the buttons
     * 
     */
    protected JMenuBar menu_bar_;

    /**
     * 
     * The Window containing the splitpane in which the Hierarchy and the
     * MiniBrowser reside
     * 
     */
    protected BrowserPanel browser_panel_;

    /**
     * Constructor sets up components and InternalFrameListener.
     * 
     * @param bdialog -
     *            The parent of this Frame
     */
    public BrowserFrameSwing(IBrowserDialog bdialog, String title) {
        super(bdialog, title);

        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);

        // / setup menu bar
        menu_bar_ = new JMenuBar();
        setJMenuBar(menu_bar_);

        browser_panel_ = new BrowserPanel(this);

        getContentPane().add(browser_panel_);
        addInternalFrameListener(this);


        map_buttons_actions_ = new HashMap<JButton, EUIElement>();
        map_actions_buttons_ = new HashMap<EUIElement, JButton>();
    }

    /***************************************************************************
     * 
     * Button events
     * 
     */

    /**
     * Main method for handling events that come from UI_Elements It is prety
     * generic, only looks up the button in the map and sends the UserAction to
     * the browser dialog.
     * 
     */
    public void actionPerformed(ActionEvent event) {
        System.out
                .println("In BrowserFrame::actionPerformed(ActionEvent event)");

        // / get the source of the event
        Object event_source = event.getSource();

        // / check if it is a button
        if (event_source instanceof JButton) {
            JButton src_button = (JButton) event_source;

            // / check the map and get the actionID
            EUIElement actionID = map_buttons_actions_.get(src_button);

            // / if the button is registered
            if (actionID != null) {
                notifyBrowserDialog(new UserAction(actionID));
            } else {
                System.out
                        .println("Got event from a button, but the button is not registered !!!");
            }
        } else {
            System.out.println("Got event from someone else!!!");
        }
    }

    /**
     * To add dynamic buttons to the menu of the frame.
     * 
     * @param action
     *            The action assigned to the button.
     */
    private void addToMenuBar(IUIAction action) {
        EUIElement action_id = action.getAction();

        // check if a button is already in the menu
        if (!map_actions_buttons_.containsKey(action_id)) {
            JButton dynamic_button = new JButton(action.getLabel());
            dynamic_button.setName(action.getLabel());
            dynamic_button.addActionListener(this);

            map_actions_buttons_.put(action_id, dynamic_button);
            map_buttons_actions_.put(dynamic_button, action_id);

            menu_bar_.add(dynamic_button);
            menu_bar_.updateUI();
        }
    }

    /**
     * Removes item from menu.
     * 
     * @param action
     * @return true if done else false
     */
    private boolean removeFromMenuBar(IUIAction action) {

        // / get the ID of the action
        EUIElement actionID = action.getAction();

        // / check if the button is registered
        if (!map_actions_buttons_.containsKey(actionID)) {
            System.out
                    .println("In BrowserFrame::removeFromMenuBar() - The button is not registered !!");
            return false;
        } else {
            JButton button_to_delete = (JButton) map_actions_buttons_
                    .get(actionID);

            map_actions_buttons_.remove(actionID);
            map_buttons_actions_.remove(button_to_delete);

            getJMenuBar().remove(button_to_delete);
            getJMenuBar().updateUI();

            return true;
        }
    }

    /***************************************************************************
     * 
     * Frame events
     * 
     */

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * If the Browser has focus, then the apropriate BrowserDialog is notified
     * so that it can retrieve the current context from the WSDialog.
     * 
     */
    public void internalFrameActivated(InternalFrameEvent arg0) {
        System.out.println("In BrowserFrame::internalFrameActivated() ");

        try {
            browser_dialog_.notifyUserAction(new UserAction(
                    EUIElement.UI_BROWSER_GOT_FOCUS,
                    EUIContext.UI_CONTEXT_BROWSER));
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (DialogProtocolException e) {
            e.printStackTrace();
        }
    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * The Browser is only closed when the Session is closed as well (logout)
     * 
     */
    public void internalFrameClosed(InternalFrameEvent arg0) {
        System.out.println("In BrowserFrame::internalFrameClosed() ");

    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * If the close button is clicked, then the frame becomes invisible, but it
     * still exists. The event UI_BROWSER_LOST_FOCUS is sent to the
     * BrowserDialog altough this event is not used by the BrowserDialog
     * 
     */
    public void internalFrameClosing(InternalFrameEvent arg0) {
        System.out.println("In BrowserFrame::internalFrameClosing() ");
        this.setVisible(false);

        try {
            browser_dialog_.notifyUserAction(new UserAction(
                    EUIElement.UI_BROWSER_LOST_FOCUS,
                    EUIContext.UI_CONTEXT_BROWSER));
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (DialogProtocolException e) {
            e.printStackTrace();
        }
    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * If the minimise button is clicked, then the frame becomes invisible, but
     * it still exists. The event UI_BROWSER_LOST_FOCUS is sent to the
     * BrowserDialog altough this event is not used by the BrowserDialog
     * 
     */
    public void internalFrameDeactivated(InternalFrameEvent arg0) {
        System.out.println("In BrowserFrame::internalFrameDeactivated() ");

    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * Im not sure whether this will ever happen
     * 
     */
    public void internalFrameDeiconified(InternalFrameEvent arg0) {

        System.out.println("In BrowserFrame::internalFrameDeiconified() ");
        System.exit(1);

    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * Im not sure whether this will ever happen
     * 
     */
    public void internalFrameIconified(InternalFrameEvent arg0) {

        System.out.println("In BrowserFrame::internalFrameIconified() ");
        System.exit(1);

    }

    /**
     * 
     * Method to listen to the state of this Frame
     * 
     * This method is called when the Browser is created. Can be used to send
     * some initialisation message to the BrowserDialog
     * 
     */
    public void internalFrameOpened(InternalFrameEvent arg0) {
        System.out.println("In BrowserFrame::internalFrameOpened() ");
    }

    /***************************************************************************
     * 
     * MiniBrowser events
     * 
     */

    /**
     * 
     * Inherited from HyperlinkListener
     * 
     * Sends the clicked link to the BrowserDialog and asks what to do
     * 
     */
    public void hyperlinkUpdate(HyperlinkEvent hyperlink_event) {
        System.out.println("In BrowserFrame::hyperlinkUpdate() : URL is "
                + hyperlink_event.getURL());
        System.out.println("In BrowserFrame::hyperlinkUpdate() : element is "
                + hyperlink_event.getSourceElement());

        if (hyperlink_event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

            UserActionOnLink uaol = new UserActionOnLink(
                    EUIElement.UI_MINIBROWSER_LINK,
                    EUIContext.UI_CONTEXT_MINIBROWSER, hyperlink_event.getURL());
            if (logger.isInfoEnabled())
                logger.info(" BF->BD: url=" + hyperlink_event.getURL());
            try {
                browser_dialog_.notifyUserAction(uaol);
            } catch (RemoteException e) {
                e.printStackTrace();
            } catch (DialogProtocolException e) {
                e.printStackTrace();
            }

        }
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * In the case of the BrowserFrame, the method should add the button
     * containing the UserAction to the main menu bar.
     * 
     */
    public void addUIElement(IUIAction action) throws RemoteException {
        System.out
                .println("In BrowserFrame::addUIElement(IUIAction action) : Action is "
                        + action.getAction()
                        + ", Context is "
                        + action.getContext());

        if (action.getContext() == EUIContext.UI_CONTEXT_BROWSER) {
            addToMenuBar(action);
        }
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * In the case of the BrowserFrame, the method should remove the button
     * containing the UserAction (if such button exists) from the main menu bar.
     * 
     */
    public boolean removeUIElement(IUIAction action) throws RemoteException {

        System.out
                .println("In BrowserFrame::doUIAction(IUIAction action) : Action is "
                        + action.getAction()
                        + ", Context is "
                        + action.getContext());

        if (action.getContext() == EUIContext.UI_CONTEXT_BROWSER) {
            return removeFromMenuBar(action);
        }

        return false;
    }

    /**
     * Method inherited from IToGUI interface
     * 
     * Performs the dispatching of the UIAction. This method is called by the
     * appropriate Dialog over RMI.
     * 
     * @param action -
     *            the action to perform. The BrowserFrame subclasses can
     *            implement following actions :
     * 
     * UIAction: UI_BROWSER_AUTO_MATCH_ON UI_BROWSER_AUTO_MATCH_OFF
     * 
     * UIActionOnMathContext: UI_BROWSER_CHANGE_CONTEXT
     * 
     * UIActionOnLink: UI_MINBROWSER_LINK
     * 
     * With the following contexts: UI_CONTEXT_HIERARCHY UI_CONTEXT_MINIBROWSER
     * UI_CONTEXT_FOCUS
     * 
     * The subclasses can call super.doUIAction( action ) to process the actions
     * with UI_CONTEXT_BROWSER context.
     * 
     * The following actions are implemented here:
     * 
     * UI_BROWSER_GET_FOCUS
     * 
     */
    public void doUIAction(IUIAction action) throws RemoteException {

        System.out
                .println("In BrowserFrame::doUIAction(IUIAction action) : Action is "
                        + action.getAction()
                        + ", Context is "
                        + action.getContext());

        switch (action.getContext()) {
        case UI_CONTEXT_CONTENT:
            // / Dispatch actions to the MiniBrowser
            browser_panel_.getMiniBrowser().doUIAction(action);
            break;
        case UI_CONTEXT_HIERARCHY:
            // / Dispatch the action to the hierarchy
            browser_panel_.getHierarchyPanel().doUIAction(action);
            break;
        default: break;
        }

        // / if it is not our context go back
        if (action.getContext() != EUIContext.UI_CONTEXT_BROWSER)
            return;

        switch (action.getAction()) {
        case UI_BROWSER_GET_FOCUS:
            this.setVisible(true);
            this.toFront();
            break;

        case UI_BROWSER_CHANGE_CONTEXT:
            UIActionOnMathContext uaomc = (UIActionOnMathContext) action;
            current_context_ = uaomc.getMathContext();
            notifyContextChangeListeners();
            break;

        default:
            System.out
                    .println("Which witch watches which switch (in BrowserFrame::doUIAction()");
            break;
        }
    }

}
