/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.calcheadviews;

import isac.util.formulae.CalcHead;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import org.apache.log4j.Logger;

/**
 * @author Mario Hochreiter
 * @author Manuel Koschuch
 * @version 1.0
 * 
 */
public class CalcHeadView extends JPanel implements ICalcHeadView {

    /**
     * Constant used for serialization (@see corresponding API Documentation)
     */
    private static final long serialVersionUID = 1L;

    private static final Logger logger = Logger.getLogger(CalcHeadView.class
            .getName());

    private static final int HEADLINE_PANEL_INDEX = 0;

    private static final int MODEL_PANEL_INDEX = 1;

    private static final int SPECIFICATION_PANEL_INDEX = 2;

    private ResourceBundle res_bundle_;

    private ModelPanel model_panel_;

    private SpecificationPanel specification_panel_;

    private HeadLinePanel head_line_panel_;

    private int current_panel_index_;

    /**
     * Creates a new CalcHeadView, consisting of 3 Panels: a HeadLinePanel,
     * presenting the CalcHead's Headline, a ModelPanel, presenting the
     * CalcHead's Model, a SpecificationPanel, presenting the CalcHead's
     * Specification
     * 
     * depending on the selected Viewstyle in the supplied CalcHead all or some
     * of these panels are drawn
     * 
     */

    public CalcHeadView(CalcHead ch) {
        res_bundle_ = ResourceBundle.getBundle("CalcHeadPanel");
        model_panel_ = new ModelPanel(ch.getModel(), res_bundle_);
        specification_panel_ = new SpecificationPanel(ch.getSpecification(),
                res_bundle_);
        head_line_panel_ = new HeadLinePanel(ch.getHeadLine());
        current_panel_index_ = 0;
        this.init(ch);
        this.fillTextFieldsFromCalcHead(ch);
    }

    private void init(CalcHead ch) {
        this.layoutComponents(ch);
    }

    private void layoutComponents(CalcHead ch) {
        this.setLayout(new BorderLayout());
        this.add(head_line_panel_, BorderLayout.NORTH);
        if (ch.getBelongsTo().equals("Met")) {
            buildMethodLayout();
        } else {
            buildProblemLayout(ch);
        }

    }

    /**
     * @param ch
     *            CalcHead to wich the data read from the text fields is written
     */
    public void fillCalcHeadFromTextFields(CalcHead ch) {
        model_panel_.fillModelFromTextFields(ch.getModel());
        specification_panel_.fillSpecificationFromTextFields(ch
                .getSpecification());
        ch.setBelongsTo(specification_panel_.getBelongsTo());
    }

    /**
     * @param ch
     *            CalcHead from wich the data is read to fill out the text
     *            fields
     */
    public void fillTextFieldsFromCalcHead(CalcHead ch) {
        if (logger.isDebugEnabled())
            logger.debug(ch.getModel().getGiven() + " "
                    + ch.getModel().getFind());

        reset();
        head_line_panel_.fillHeadLinePanelFromFormula(ch.getHeadLine());
        model_panel_.fillTextFieldsFromModel(ch.getModel(), true);
        specification_panel_.fillTextFieldsFromSpecification(ch
                .getSpecification(), ch.getBelongsTo());
    }

    private void buildMethodLayout() {
        // FIXME find better representation?
        specification_panel_.highlightLabel(SpecificationPanel.LABEL_METHOD);
        buildFullLayout("Guard");
    }

    private void buildProblemLayout(CalcHead ch) {
        specification_panel_.highlightLabel(SpecificationPanel.LABEL_PROBLEM);
        switch (ch.getViewstyle()) {
        case CalcHead.VIEWSTYLE_FULL:
            buildFullLayout("Model");
            break;
        case CalcHead.VIEWSTYLE_MODEL:
            buildModelLayout();
            break;
        case CalcHead.VIEWSTYLE_SPECIFICATION:
            buildSpecificationLayout();
            break;
        default:
            buildFullLayout("Model");
        }
    }

    /**
     * Builds ModelPanel and SpecificationPanel, the ModelPanel is titled with
     * the supplied string (either Model or Guard)
     * 
     * @param title
     *            title for the ModelPanel
     */

    private void buildFullLayout(String title) {
        this.add(model_panel_, BorderLayout.CENTER);
        model_panel_.setBorder(BorderFactory.createTitledBorder(title));

        this.add(specification_panel_, BorderLayout.SOUTH);
        specification_panel_.setBorder(BorderFactory
                .createTitledBorder("Specification"));

    }

    /**
     * Build only ModelPanel
     * 
     */

    private void buildModelLayout() {
        this.add(model_panel_, BorderLayout.CENTER);
        model_panel_.setBorder(BorderFactory.createTitledBorder("Model"));

    }

    /**
     * Build only SpecificationPanel
     * 
     */

    private void buildSpecificationLayout() {
        this.add(specification_panel_, BorderLayout.CENTER);
        specification_panel_.setBorder(BorderFactory
                .createTitledBorder("Specification"));

    }

    public ModelPanel getModelPanel() {
        return model_panel_;
    }

    public SpecificationPanel getSpecificationPanel() {
        return specification_panel_;
    }

    /**
     * Clears all fields
     */

    public void reset() {
        current_panel_index_ = 0;
        head_line_panel_.reset();
        model_panel_.reset();
        specification_panel_.reset();

    }

    /**
     * Fills Textfields stepwise
     * 
     * @param ch
     *            CalcHead to fill TextFields from
     * @return false, when Textfields left to fill, true otherwise
     * @deprecated was misunderstanding
     */

    public boolean next(CalcHead ch) {
        switch (current_panel_index_) {
        case HEADLINE_PANEL_INDEX:
            if (head_line_panel_.next(ch.getHeadLine())) {
                current_panel_index_++;
                return false;
            }
            current_panel_index_++;
            return false;

        case MODEL_PANEL_INDEX:
            if (model_panel_.next(ch.getModel())) {
                current_panel_index_++;
                return false;
            }
            return false;

        case SPECIFICATION_PANEL_INDEX:
            if (specification_panel_.next(ch.getSpecification())) {
                current_panel_index_++;
                return true;
            }
            return false;

        default:
            return true;
        }
    }
}