package isac.gui.mawen.editor

import isac.gui.mawen.syntax.Ast._
import isac.gui.mawen.syntax.Ast

import isac.gui.mawen.syntax.AstException
import isac.gui.mawen.syntax.XSyntax

/**
 * Utilities to retrieve details from an <code>Ast</code>
 */
object AstInfoUtil {
  
  def ignoreBox(ast: Ast) : Ast = ast match {
    case Appl(List(Constant(str), ast)) if str.startsWith("BOX") => ast
    case a                                                       => a
  }
  def astOfBox(ast: Ast) : Ast = ast match {
    case Appl(List(Constant(str), a)) if str.startsWith("BOX")   => a
    case _ => throw new AstException("astOfBox, without Box: "  + Ast.raw_string_of(ast))
  }
  def isBox (ast: Ast) : Boolean = ast match {
    case Appl(List(Constant(str), ast)) if str.startsWith("BOX") => true
    case _                                                       => false
  }
  def astOfCursor(ast: Ast) : Ast = ast match {
    case Appl(List(Constant("CURSOR"), a))                       => a
    case Appl(List(a, Constant("CURSOR")))                       => a
    case _ => throw new AstException("astOfBox, without Box: "  + Ast.raw_string_of(ast))
  }
  def isCursor (ast: Ast) : Boolean = ast match {
    case Appl(List(Constant("CURSOR"), ast))                      => true
    case Appl(List(ast, Constant("CURSOR")))                      => true
    case _                                                       => false
    
  }
  def FirstChild(ast: Ast) : Option[Ast] = ast match {
    case Appl(asts) => Some(asts.head)
    case _ => None
  }
  def NextChild(parent: Ast, ast: Ast) : Ast = parent match {
    case Appl(asts) => {
      var prev = asts.head
      for(a <- asts.tail :+ asts.head) {
        if (ast eq prev) {
          return a
        }
        prev = a
      }
      return ast
    }
    case _ => ast
  }
  def PrevChild(parent: Ast, ast: Ast) : Ast = parent match {
    case Appl(asts) => {
      var prev = asts.head
      for(a <- asts.tail :+ asts.head) {
        if (ast eq a) {
          return prev
        }
        prev = a
      }
      return ast
    }
    case _ => ast
  }
  def Parent(root: Ast, ast: Ast) : Option[Ast] = root match {
    case Appl(asts) => {
      for (a <- asts) {
        if (a eq ast) {
          return Some(root)
        }
      }
      for (a <- asts) {
        Parent(a, ast) match {
          case Some(a) => return Some(a)
          case None => {}
        }
      }      
      return None
    }
    case _ => None
  }
  def hasBox(ast: Ast) : Boolean = ast match {
    case Constant(str) if str.startsWith("BOX") => true
    case Constant(str) => false
    case Variable(str) => false
    case Appl(asts) => asts.exists(x => hasBox(x))
  }
  def hasCursor(ast: Ast) : Boolean = ast match {
    case Constant("CURSOR")  => true
    case Constant(str)  => false
    case Variable(str) => false
    case Appl(asts) => asts.exists(x => hasBox(x))
  }
  def FindBox(ast: Ast) : Option[Ast] = ast match {
     case Appl(List(Constant(str), a)) if str.startsWith("BOX") => Some(ast)
     case Appl(asts)                                            => {
       val l = asts.map(x => FindBox(x))
       l.foreach(x => x match {
         case Some(a) => return x
         case None => {}
       })
       if (l != null && l.length >0) {
         return l.head
       }
       return None
     }
     case ast                                                   => None
  }
  // precondition: there is only ONE "CURSOR" in a formula.
  // introducing Option[Ast] would cause delicate changes
  def FindCursor(ast: Ast) : Ast = ast match {
     case Appl(List(Constant("CURSOR"), a))                               => ast
     case Appl(List(a, Constant("CURSOR")))                               => ast
     case Appl(asts)                                                      => {
       val l = asts.map(x => FindCursor(x)).filter(y => y != null)
       if (l != null && l.length > 0) {
         return l.head
       }
       return null
     }
     case ast                                                             => null
  }
  // introducing Option[Ast] would cause delicate changes
  def AstOfCursor(ast: Ast) : Ast = ast match {
    case Appl(List(Constant("CURSOR"), a))           => a
    case Appl(List(a, Constant("CURSOR")))           => a
    case _ => null
  }
  def OperatorString(ast: Ast) : String = ast match {
    case Constant(str) => str
    case a if AstInfoUtil.isBox(a) => OperatorString(AstInfoUtil.astOfBox(a))
    case a if AstInfoUtil.isCursor(a) => OperatorString(AstInfoUtil.astOfCursor(a))
    case _ => throw new AstException("OperatorString, uncovered match: " + raw_string_of(ast))
  }
  def VariableString(ast: Ast) : String = ast match {
    case Variable(str) => str
    case _ => throw new AstException("VariableString, uncovered match: " + raw_string_of(ast))
  }
  def priorOfAst(ast: Ast) : Int = ast match {
    case Constant("CURSOR") => 0
    case Constant("GAP") => 0
    case Variable(str) => 0
    case Constant(str) => XSyntax.isa_prior(str)
    case Appl(asts) => asts.map(x => priorOfAst(x)).max
  }
}