package isac.gui.mawen.editor

import isac.gui.mawen.editor.Box._
import isac.gui.mawen.syntax.Ast._
import isac.gui.mawen.syntax.Ast.raw_string_of
import isac.gui.mawen.syntax.AstException
import isac.gui.mawen.syntax.XSyntax
import java.awt.Graphics
import java.awt.Font
import java.awt.Rectangle
import java.awt.Graphics2D
import java.awt.Color
import javax.swing.JComponent
import javax.swing.Spring.HeightSpring

/**
 * Calculate a <code>Box</code> for each leaf
 * of an <code>Ast</code> and assemble them bottom up according to
 * layouts for specific structures of sub-asts.
 */
object CalcUtil {
  
  val emptyFunc = (box: Box) => {}
  def assembleComponent(ast: Ast): JComponent = new AstComponent(ast)
  def assembleBoxes (ast: Ast, f: (DrawBox) => Unit ) =
    assembleBoxesRec(new EmptyBox(0,0,0,ast), f)

  /*
   * Recursively assemble boxes, i.e. sub-terms of a formula while scanning the Ast.
   * 
   * For the call back see  $4.3.1 p.53/54 and $4.3.2 p.56.
   */
  def assembleBoxesRec(emptyBox: EmptyBox, callback_calc:(DrawBox) => Unit) : DrawBox = emptyBox.ast match {
    case Appl(operator :: param :: Nil)
      => assembleUniopBoxes(EmptyUniOpBox(emptyBox.level, emptyBox.x0, emptyBox.y0, param, emptyBox.ast), callback_calc)
    case Appl(operator :: paraml :: paramr :: Nil)
      => assembleBinopBoxes(AstInfoUtil.OperatorString(operator), EmptyBinOpBox(emptyBox.level,emptyBox.x0, emptyBox.y0, operator, paraml, paramr,emptyBox.ast), callback_calc)
    case Appl(Constant(str) :: param)
      => calcDefault(str,param, emptyBox.x0, emptyBox.y0, emptyBox.level, emptyBox.ast, callback_calc)
    case a
      => assembleLeafBoxes(EmptyLeafOpBox(emptyBox.level, emptyBox.x0, emptyBox.y0, emptyBox.ast), callback_calc)
  }
  def assembleLeafBoxes(emptyBox: EmptyLeafOpBox, callback_calc:(DrawBox) => Unit) : DrawBox = emptyBox.ast match {
    case Variable("GAP") => calcGAP( emptyBox, callback_calc)
    case Constant("GAP") => calcGAP( emptyBox, callback_calc)
    case Constant(str)   => calcConstant( emptyBox, callback_calc)
    case Variable(str)   => calcVariable( emptyBox, callback_calc)
    case ast => throw new AstException("assembleBoxesRec, uncovered match: " + raw_string_of(ast))
  }
  def assembleUniopBoxes(emptyBox: EmptyUniOpBox, callback_calc:(DrawBox) => Unit) : DrawBox = emptyBox.ast match {
    case Appl(Constant(str) ::  ast :: Nil) if str.startsWith("BOX")
      => CalcBox( str.replace("BOX.", ""), emptyBox, callback_calc)
    case Appl(Constant("CURSOR") :: param :: Nil)
      => calcCursor( EmptyUniOpBox(emptyBox.level,emptyBox.x0,emptyBox.y0, param, emptyBox.ast), callback_calc)
    case Appl(param :: Constant("CURSOR") :: Nil)
      => calcCursor( EmptyUniOpBox(emptyBox.level,emptyBox.x0,emptyBox.y0, param, emptyBox.ast), callback_calc)
    case Appl(op :: param :: Nil)
      => calcDefault( AstInfoUtil.OperatorString(op), List(param), emptyBox.x0, emptyBox.y0, emptyBox.level, emptyBox.ast, callback_calc)
    case ast
      => throw new AstException("assembleBoxesRec, uncovered match: " + raw_string_of(ast))
  }
  def assembleBinopBoxes( operatorString: String, emptyBox: EmptyBinOpBox, callback_calc:(DrawBox) => Unit) : DrawBox = Settings.layoutName(XSyntax.isa_ast(operatorString) ) match {
    case "binOp" => calcDefaultBinOp( emptyBox, callback_calc)
    case "Pow"   => calcPow( emptyBox, callback_calc)
    case "Frac"  => calcFrac( emptyBox, callback_calc)
    case _       => calcDefault( operatorString, List(emptyBox.paramr, emptyBox.paraml), emptyBox.x0, emptyBox.y0, emptyBox.level, emptyBox.ast, callback_calc)
  }
  /*
   * calc* calculates the size of a frame around <code>DrawBox</code>.
   * 
   * For the call back see  $4.3.1 p.53/54 and $4.3.2 p.56.
   */
  def calcVariable( emptybox: EmptyLeafOpBox, callback_calc:(DrawBox) => Unit): DrawBox = {
    val level = emptybox.level
    val x0 = emptybox.x0
    val y0 = emptybox.y0
    val ast =  emptybox.ast
    val str = AstInfoUtil.VariableString(emptybox.ast)
    val box : Rectangle =  Settings.getStringBoundOf(str, level, x0, y0) //  getStringBounds(g.asInstanceOf[Graphics2D], str, x0, y0) // TODO: Why does this work without x0,y0?
    val b = DrawBox(str, level, box.x, box.y + box.height, box.width, box.height, x0, y0, ast)
    callback_calc(b)
    b
  }
  def calcConstant( emptyBox: EmptyLeafOpBox, callback_calc:(DrawBox) => Unit) = {
    val level = emptyBox.level
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val ast =  emptyBox.ast
    val op = XSyntax.isa_ast((AstInfoUtil.OperatorString(emptyBox.ast)))
    val box : Rectangle = Settings.getStringBoundOf(op, level,x0,y0)  //getStringBounds(g.asInstanceOf[Graphics2D], op, x0, y0)
    val midline = y0 - (fontsizeOf(level) * (2f / 3f)*(1f / 3f)).round
    val newY = ((box.y - y0) * -1) - box.height /2 + midline
    val b = DrawBox(op, level, box.x, y0, box.width, fontWidthOf(fontsizeOf(level)),x0, newY, ast)
    callback_calc(b)
    b
  }
  def calcGAP( emptyBox: EmptyLeafOpBox, callback_calc:(DrawBox) => Unit) = {
    val level = emptyBox.level
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val ast =  emptyBox.ast
    val box : Rectangle = Settings.getStringBoundOf("xx", level, x0, y0) //getStringBounds(g.asInstanceOf[Graphics2D], "xx", x0, y0)
    val b = new DrawBox("GAP", level, box.x, y0, box.width, fontWidthOf(fontsizeOf(level)),x0, y0, ast)
    callback_calc(b)
    b
  }
  // NO callback_calc, because a frame does NOT accept a mouse interaction
  def CalcBox( str: String, emptyBox: EmptyUniOpBox, f:(DrawBox) => Unit) = {
     val box = assembleBoxesRec( new EmptyBox(emptyBox.level, emptyBox.x0, emptyBox.y0, emptyBox.param), f)
     box.isBoxed = true
     if (str.split('.').head == "MAIN") {
       box.boxedElement = str.split('.').tail.head.toInt
       box.colorCode = 0
     }else {
       box.boxedElement = 0
       box.colorCode = str.split('.').head.toInt
     }
     box
  }
  def calcCursor( emptyBox: EmptyUniOpBox, callback_calc:(DrawBox) => Unit) = {
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val level = emptyBox.level
    val ast = emptyBox.ast
    val param = emptyBox.param
    val childBox = assembleBoxesRec( new EmptyBox(level, x0, y0, param), callback_calc)
    val b = new DrawBox("CURSOR", level, childBox.x - 1, childBox.y + 1, childBox.width + 2,  childBox.height + 2, x0, y0, ast)
    callback_calc(b)
    b.children = List(childBox)
    b
  }
  /*
   * layout of horizontal type +, (binary)-, *, =
   */
  def calcDefaultBinOp( emptyBox: EmptyBinOpBox, callback_calc:(DrawBox) => Unit) = {
    val operator = emptyBox.operator
    val paraml = emptyBox.paraml
    val paramr = emptyBox.paramr
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val level= emptyBox.level
    val ast = emptyBox.ast
    val horizontalCloseBracketSpace = Settings.getStringBoundOf(")", level, x0, y0) //getStringBounds(g.asInstanceOf[Graphics2D], ")", x0, y0)
    val constantSpace = Settings.getStringBoundOf("(i", level, x0, y0)
    val (leftSpace:Int, op:String, rightSpace:Int) = {
      val str = Settings.layoutStyle(XSyntax.isa_ast(AstInfoUtil.OperatorString(operator)))
      (Settings.SpaceOf(str.charAt(0)), str.charAt(1).toString(), Settings.SpaceOf(str.charAt(2)))
    }
    var lspaceOpen  = 0
    var lspaceClose = 0
    var rspaceOpen  = 0
    var rspaceClose = 0
    if (AstInfoUtil.priorOfAst(paraml) != 0 && AstInfoUtil.priorOfAst(paraml) < AstInfoUtil.priorOfAst(ast)) {
      lspaceOpen = constantSpace.width
      lspaceClose  = horizontalCloseBracketSpace.width
    }
    if (AstInfoUtil.priorOfAst(paramr) != 0 && AstInfoUtil.priorOfAst(paramr) < AstInfoUtil.priorOfAst(ast)) {
      rspaceOpen = constantSpace.width
      rspaceClose  = horizontalCloseBracketSpace.width
    }
    val l_box = assembleBoxesRec( emptyBox.toEmptyBox(x0 + lspaceOpen, y0, paraml), callback_calc)
    val l_width = l_box.width
    val l_height = l_box.height
    
    val o_box = assembleBoxesRec( emptyBox.toEmptyBox(x0 + l_width + leftSpace + 
          lspaceOpen + lspaceClose, y0, operator), callback_calc)
    val operator_width = o_box.width
    val operator_height  = o_box.height
    
    val r_box = assembleBoxesRec( emptyBox.toEmptyBox(x0 + l_width + leftSpace + operator_width + 
          rightSpace + lspaceOpen + lspaceClose + rspaceOpen, y0, paramr), callback_calc)
    val r_width = r_box.width
    val r_height = r_box.height
    
    val newX = x0
    val newY = Math.max(l_box.y, r_box.y)
    val newWidth = l_width + leftSpace + operator_width + rightSpace + r_width + lspaceOpen + lspaceClose
    val newHeight = Math.max(l_height, r_height)

    val b = new DrawBox("", level,
        newX, newY,
        newWidth, newHeight, 
        x0, y0, ast)
    b.children =  o_box :: l_box :: r_box :: List.empty[DrawBox]
    
    if (AstInfoUtil.priorOfAst(paraml) != 0 && AstInfoUtil.priorOfAst(paraml) < AstInfoUtil.priorOfAst(ast)) {
      val openBracketL =  new DrawBox("(", level, x0, y0, constantSpace.width, constantSpace.height, x0, y0, null)
      val closeBracketL =  new DrawBox(")", level, x0 + l_width + lspaceOpen, y0, constantSpace.width, constantSpace.height, x0 + l_width + lspaceOpen, y0, null)
      b.children =  openBracketL :: closeBracketL :: b.children
    }
    if (AstInfoUtil.priorOfAst(paramr) != 0 && AstInfoUtil.priorOfAst(paramr) < AstInfoUtil.priorOfAst(ast)) {
      val openBracketR =  new DrawBox("(", level, 
          x0 + l_width + Settings.MEDIUMSPACE + operator_width + Settings.MEDIUMSPACE + lspaceOpen + lspaceClose, r_box.y0, 
          constantSpace.width, constantSpace.height, 
          x0 + l_width + Settings.MEDIUMSPACE + operator_width + Settings.MEDIUMSPACE + lspaceOpen + lspaceClose, y0, null)
      
      val closeBracketR =  new DrawBox(")", level, 
          x0 + l_width + Settings.MEDIUMSPACE + operator_width + Settings.MEDIUMSPACE + lspaceOpen + lspaceClose + rspaceOpen + r_width, r_box.y0, 
          constantSpace.width, constantSpace.height, 
          x0 + l_width + Settings.MEDIUMSPACE + operator_width + Settings.MEDIUMSPACE + lspaceOpen + lspaceClose + rspaceOpen + r_width, y0, null)
     
      b.children =  openBracketR :: closeBracketR :: b.children
    }
    callback_calc(b) // see mmahringer $4.3.1 p.53/54 and $4.3.2 p.56
    b
  }
  /*
   * layout of vertical type ^
   */
  def calcPow( emptyBox: EmptyBinOpBox, callback_calc:(DrawBox) => Unit) = {
    val operator = emptyBox.operator
    val paraml = emptyBox.paraml
    val paramr = emptyBox.paramr
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val level= emptyBox.level
    val ast = emptyBox.ast
    
    val l_box = assembleBoxesRec( new EmptyBox(level, x0, y0, paraml), callback_calc)
    val l_width = l_box.width
    val l_height = l_box.height

    val r_box = assembleBoxesRec( new EmptyBox(level + 1, x0 + l_width + 
          Settings.POWHORIZONTALSPACE, y0 - (l_height * (0.7f + level / 10f)).round, paramr), callback_calc)
    val r_width = r_box.width
    val r_height= r_box.height
    
    val b = DrawBox ("", level, x0, y0, l_width + Settings.POWHORIZONTALSPACE + r_width, l_height + r_height,x0, y0,ast)
    b.children =  l_box :: r_box :: List.empty[DrawBox]
    callback_calc(b) // see mmahringer $4.3.1 p.53/54 and $4.3.2 p.56
    b
  }
  
  /*
   * layout of vertical type with horizontal fraction bar
   */
  def calcFrac( emptyBox: EmptyBinOpBox, callback_calc:(DrawBox) => Unit) = {
    val operator = emptyBox.operator
    val paramUpper = emptyBox.paraml
    val paramLower = emptyBox.paramr
    var x0 = emptyBox.x0
    var y0 = emptyBox.y0
    val level= emptyBox.level
    val ast = emptyBox.ast

    val u_box = assembleBoxesRec( new EmptyBox(level+1, x0, y0, paramUpper), emptyFunc)
    val l_box = assembleBoxesRec( new EmptyBox(level+1, x0, y0, paramLower), emptyFunc)
    
    var difY0lower = 0
    if (lowestY0(u_box) > y0) {
      difY0lower = lowestY0(u_box) - y0
    }
    var difY0upper = 0
    if (highestY0(l_box) < y0) {
      difY0upper = y0 - highestY0(l_box)
    }
    val y_midline = y0 - (fontsizeOf(level) * (2f / 3f)*(1f / 3f)).round
    
    val maxWidth = Math.max(u_box.width, l_box.width)
    val line_box = new DrawBox("Fields.inverse_class.divide", level, x0, y_midline, maxWidth, 3, x0, y0, ast);
    // calc position for upper Frac
    val u_x0 = x0 + (maxWidth / 2f - u_box.width / 2f).round
    val u_y0 = y_midline - Settings.FRACVERTIKALSPACE - difY0lower
    // calc position for lower Frac
    val l_x0 = x0 + (maxWidth / 2f - l_box.width / 2f).round
    val l_y0 = y_midline + Settings.FRACVERTIKALSPACE + fontWidthOf(fontsizeOf(level)) + difY0upper
    
    val u_res_box = assembleBoxesRec( new EmptyBox(level+1,  u_x0, u_y0,paramUpper), callback_calc)
    val l_res_box = assembleBoxesRec( new EmptyBox(level+1, l_x0, l_y0, paramLower), callback_calc)
    
    val b = new DrawBox(
        "", level, x0, l_res_box.y, maxWidth, 
        u_res_box.height + l_res_box.height + line_box.height + Settings.FRACVERTIKALSPACE + Settings.FRACVERTIKALSPACE, 
        x0, y0, ast)
    b.children = line_box :: u_res_box :: l_res_box :: List.empty[DrawBox]
    callback_calc(b) // see mmahringer $4.3.1 p.53/54 and $4.3.2 p.56
    b
  }
  // default for rendering unknown Asts (tuple, list, ...)
  def calcDefault( str: String, param: List[Ast], x0: Int, y0: Int, level: Int, ast : Ast, callback_calc:(DrawBox) => Unit) = {
    val boxes = param.map(x => assembleBoxesRec( new EmptyBox(level, x0, y0, x),emptyFunc))
    
    val horizontalSpace =             Settings.getStringBoundOf(",x", level, x0, y0)
    val horizontalCloseBracketSpace = Settings.getStringBoundOf("i)", level, x0, y0)
    val constant =               Settings.getStringBoundOf(XSyntax.isa_ast(str), level, x0, y0)
    val constantSpace = Settings.getStringBoundOf("(i", level, x0, y0)
   
    var x = x0 + constantSpace.width + constant.width
    var y = y0
    var resBoxes = List.empty[DrawBox]

    for (box <- boxes) {
      val b = assembleBoxesRec( new EmptyBox(level, x, y, box.ast), callback_calc)
      resBoxes = b :: resBoxes 
      if (box == boxes.last) {
        resBoxes =  new DrawBox(")", level, b.x,b.y,horizontalSpace.width,horizontalSpace.height,b.x + b.width,y,null) :: resBoxes
        x = x  + horizontalCloseBracketSpace.width +  b.width
      } else {
        resBoxes =  new DrawBox(", ", level, b.x,b.y,horizontalSpace.width,horizontalSpace.height,b.x + b.width,b.y,null) :: resBoxes
        x = x + horizontalSpace.width  + b.width
      }
    }
    val resultBox = new DrawBox(XSyntax.isa_ast(str) + "(", level, x0, y0, x - x0 , boxes.map(mb => mb.height).max, x0, y0, ast);
    resultBox.children = resBoxes
    callback_calc(resultBox) // see mmahringer $4.3.1 p.53/54 and $4.3.2 p.56
    resultBox
  }
  // different from calcDefaultBinOp due to different spaces
  def calcEqualation( emptyBox: EmptyBinOpBox, callback_calc:(DrawBox) => Unit) = {
    val operator = emptyBox.operator
    val paraml = emptyBox.paraml
    val paramr = emptyBox.paramr
    val x0 = emptyBox.x0
    val y0 = emptyBox.y0
    val level= emptyBox.level
    val ast = emptyBox.ast

    val l_box = assembleBoxesRec( new EmptyBox(level, x0, y0, paraml), callback_calc)
    val l_width = l_box.width
    val l_height = l_box.height

    val o_box = assembleBoxesRec( new EmptyBox(level, x0 + l_width + 
          Settings.LARGESPACE, y0, Constant("=")), callback_calc)
    val operator_width = o_box.width
    val operator_height  = o_box.height
    
    val r_box = assembleBoxesRec( new EmptyBox(level, x0 + l_width + 
          Settings.LARGESPACE + operator_width + Settings.LARGESPACE, y0, paramr), callback_calc)
    val r_width = r_box.width
    val r_height = r_box.height

    val b = new DrawBox("", level, x0, y0, l_width + Settings.LARGESPACE + operator_width + Settings.LARGESPACE + r_width, Math.max(l_height, r_height),x0, y0, ast)
    b.children =  o_box :: l_box :: r_box :: List.empty[DrawBox]
    callback_calc(b) // see mmahringer $4.3.1 p.53/54 and $4.3.2 p.56
    b
  }
  def lowestY0(b: DrawBox): Int = {
    if (b.children.isEmpty) {
      return b.y0
    }
    var maxY0 = b.y0
    for(c <- b.children) {
      maxY0 = Math.max(Math.max(maxY0, c.y0), lowestY0(c))
    }
    return maxY0
  }
  def highestY0(b: DrawBox): Int = {
    if (b.children.isEmpty) {
      return b.y0
    }
    var maxY0 = b.y0
    for(c <- b.children) {
      maxY0 = Math.min(Math.min(maxY0, c.y0), lowestY0(c))
    }
    return maxY0
  }
  def fontWidthOf(fontsize: Int) = ((fontsize * 2f) / 3f).round

  def decreaseFontOfLevel(fontsize: Int, level: Int) = level match {
    case 1 => (fontsize * 0.7f).round
    case 2 => (fontsize * 0.7f).round
    case _ => fontsize // higher levels stay with the same
  }
  def fontsizeOf(level: Int) = level match {
    case 0 => Settings.INITIALFONTSIZE
    case 1 => (Settings.INITIALFONTSIZE * 0.7f).round
    case _ => (Settings.INITIALFONTSIZE * 0.7f * 0.7f).round
  }

}
