/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.treetable;

import isac.interfaces.ICalcIterator;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.tree.TreePath;

/**
 * CalcModelHierarchy is the "glue" between the <code>CalculationModel</code>
 * and the <code>CalcIterator</code>. This class holds Objects from type
 * <code>CalcModelNode</code> and is responsible to build up a hierarchy_ that
 * represents the <code>CalcElement</code>'s in the <code>CalcTree</code>.
 * 
 * @author Mario Hochreiter 2004
 * @version 1.0
 * 
 * @author Alois Kirchsteiger 2005
 * @version 1.1
 */

public class CalcModelHierarchy implements Serializable {
	
	static final long serialVersionUID = 5364237097682078620L;

	CalcModelNode root_;
	
	CalcModelNode insertionNode_;
	CalcModelNode lastInsertionParent_;

	/**
	 * @author Alois Kirchsteiger creates the hierachy the nodes are stored in
	 *         an ArrayList together with their levels
	 */
	public CalcModelHierarchy() {
		root_ = new CalcModelNode((ICalcIterator)null); //root of the worksheet, not visible
		lastInsertionParent_ = root_;
		insertionNode_ = null;
		// for the user
	}

	/**
	 * @return the root of the worksheet
	 */
	public CalcModelNode getRoot() {
		return root_;
	}

	/**
	 * @author Nebojsa Simic
	 * @return Object[3] for treeEvents the result has the following structure:
	 *         result[0] = path to the parent result[1] = int[1] with the index
	 *         of the inserted Node as only element result[2] = Object[1] with
	 *         the inserted Node as only element
	 * 
	 * returns null if unsuccessful, e.g. because the node is already in the
	 * tree
	 */
	public Object[] insert( CalcFormula formula ) {

		CalcModelNode parent = null;
		CalcModelNode new_node = new CalcModelNode(formula);
		
		parent = getParent( formula, root_ );
		
		if( parent == null )
		{
			System.out.println( "Parent Not Found "  );
			return null;
		}
		
		if( !parent.add(new_node) )
			return null;
		
		return new Object[] { parent.getPath().toArray(),
				new int[] { new_node.getIndex() }, new Object[] { new_node } };		
	}

	/**
	 * Deletes nodes that lie between "from" and "to"... "From" node is not deleted...
	 * 
	 * @author Nebojsa Simic
	 * @return Object[3] for treeEvents the result has the following structure:
	 *         result[0] = path to the parent result[1] = int[1] with the index
	 *         of the inserted Node as only element result[2] = Object[1] with
	 *         the inserted Node as only element
	 * 
	 * returns null if unsuccessful, e.g. because the node does not exist
	 */
	public Object[] deleteFromTo(Position from, Position to) {
	
		if ( from == null || to == null ) {
			System.out.println("Null params");
			return null;
		}
		
	
		ArrayList<Integer> return_indices = new ArrayList<Integer>();
		ArrayList<CalcModelNode> return_parents = new ArrayList<CalcModelNode>();
		ArrayList<CalcModelNode> return_nodes = new ArrayList<CalcModelNode>();
		
		Object[] nodes = getNodesFromTo(from, to);
		
		System.out.println( "Got n = " + nodes.length );
		
		for( int i = 0; i < nodes.length; i++ ) { 	
			CalcModelNode currentNode = (CalcModelNode) nodes[i];
			CalcModelNode parent = currentNode.getParent();
			int index = currentNode.getIndex();

			return_indices.add(i, new Integer(index) );
			return_nodes.add(i, currentNode );
			return_parents.add(i, parent );
		}

		for( int i = 0; i < nodes.length; i++ ) { 	
			CalcModelNode currentNode = (CalcModelNode) nodes[i];
			CalcModelNode parent = currentNode.getParent();
			
			System.out.println( "Deleting " + currentNode.getPosition().toString() );
			
			parent.remove(currentNode);
		}
			
		return new Object[] { return_parents.toArray(), return_indices.toArray(), return_nodes.toArray() }; 
	}	
	
	
	/**
	 * @param formula
	 * @return the parent node under which the new node with this formula will
	 *         be inserted
	 */
	
	private CalcModelNode getParent( CalcFormula formula, CalcModelNode node )
	{
		Position target_position = formula.getPosition();
		int target_level = target_position.getLevel();
		
		if( target_level < 1 ) // asymptotic case
			return root_; 

		Position current_position = node.getPosition();
		
		if( current_position != null )
			if( target_level == current_position.getLevel() + 1 )
				return node;
		
		Vector<CalcModelNode> children_of_node = node.getChildren();
		Iterator<CalcModelNode> children_iterator = children_of_node.iterator();

		CalcModelNode previous_node = null;
		CalcModelNode current_child = null;
		
		while( children_iterator.hasNext() ) {
				current_child = (CalcModelNode) children_iterator.next();
				
				Position current_child_position = current_child.getPosition();
				
				if( current_child_position != null )
				{				
					if( target_position.compareTo(current_child_position) < 0 )
					{
						if( previous_node != null )
							return getParent( formula, previous_node );
						else
							return getParent( formula, current_child );
					}
					previous_node = current_child;
				}
			}

		if( previous_node != null )
			return getParent( formula, previous_node );

		return null;
	}
	

	/**
	 * @param node
	 * @return the number of childs for the node
	 */
	public int getChildCount(CalcModelNode node) {
		return node.getChildCount();
	}

	/**
	 * @param elem - Position of the node whose follower we are searching 
	 * @return the next node which follows the specified value, null if node not found
	 */
	private CalcModelNode getNextNodeForValue(Position elem, CalcModelNode node) {

		CalcModelNode nde = getNodeForValue( elem, node );
		
		CalcModelNode child = null;
		CalcModelNode parent = null;
		
		if( nde == null ) {
			return null;
		}

		child = nde.getNextChild( null );
		parent = nde.getParent();
			
		if( child == null )
			child = nde.getNextNode();

		if( child == null )
		{
			if( parent != null ) {
				child = parent.getNextNode();
				if( child == null ) {
					return null;
				}
			}
			else {
				return null;
			}
		}
		return child;
	}
	
	private CalcModelNode getLastNode( CalcModelNode startNode ) {
		if( startNode == null )
			return null;
		
		if( startNode.getChildCount() == 0 )
			return startNode;
		
		Vector<CalcModelNode> startNodeChildren = startNode.getChildren();
		CalcModelNode lastChild  = (CalcModelNode) startNodeChildren.lastElement();
		
		return getLastNode( lastChild );
	}
	
	/**
	 * @param elem - Position of the node we need to find
	 * @param node - the node from where we start the search
	 * @return the node from the specified value, null if node not found
	 */
	public CalcModelNode getNodeForValue(Position elem, CalcModelNode node) {
	
		if (node.getPosition() != null && node.getPosition().equals( elem  ) ) {
			return node;
		}
		
		CalcModelNode result = null;
		Iterator<?> childIterator = node.getChildren().iterator();
		while (childIterator.hasNext()) {
			CalcModelNode currentNode = (CalcModelNode) childIterator.next();
			result = getNodeForValue( elem, currentNode);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/**
	 * @param from
	 * @param to 
	 * @return the nodes that lie in the half-open interval (from, to] 
	 */	
	public Object[] getNodesFromTo( Position from, Position to ) {
		ArrayList<CalcModelNode> result = new ArrayList<CalcModelNode>();
		Position currentPosition = (Position) from.clone();
		
		CalcModelNode node = root_;
		do {
			node = getNextNodeForValue( currentPosition, node );
			if( node != null && node.getPosition() != null ) {
				result.add( node );
				currentPosition = (Position) node.getPosition().clone();
			}
		} while( node != null && node.getPosition() != null && to.compareTo(currentPosition) != 0);

		return result.toArray();
	}
	
	/***
	 * The method returns the TreePath from the root node to the given node
	 */
	public TreePath getPathToNode( CalcModelNode node ) {
		Vector<CalcModelNode> result = new Vector<CalcModelNode>();

		result.add( 0, node );

		do {
			node = node.getParent();
			
			if( node != null ) 
				result.add( 0, node );
			
		} while( node != null && node.getPosition() != null );

		return new TreePath( result.toArray() );
	}
	
	/**
	 * resets the tree
	 */
	public void reset() {
		root_ = null;
		root_ = new CalcModelNode((ICalcIterator)null);
		lastInsertionParent_ = root_;
	}

	/**
	 * @author Alois Kirchsteiger
	 * @return Object[3] for treeEvents with an empty node the result has the
	 *         following structure: result[0] = path to the parent result[1] =
	 *         int[1] with the index of the inserted Node as only element
	 *         result[2] = Object[1] with the inserted Node as only element
	 * 
	 * this code was created for the insertion of an empty node at the end of
	 * the worksheet
	 * WN160427 seems superfluous
	 * @see isac.gui.treetable.CalculationModel
	 */
	public Object[] addInsertionNode() {
		System.out.println("Adding insertion node");
		lastInsertionParent_ = root_;
		insertionNode_ = new CalcModelNode((ICalcIterator)null);
		
		int insertionIndex = lastInsertionParent_.getChildCount();
		lastInsertionParent_.add(insertionIndex, insertionNode_);
		
		System.out.println("Added insertion node to " + insertionIndex );
		
		return new Object[] { new Object[] { root_ },
				new int[] { insertionIndex - 1 }, new Object[] { insertionNode_ } };
	}
	
	/**
	 * @return Object[3] for treeEvents with an empty node the result has the
	 *         following structure: result[0] = path to the parent result[1] =
	 *         int[1] with the index of the inserted Node as only element
	 *         result[2] = Object[1] with the inserted Node as only element
	 * 
	 * this code was created for the insertion of an empty node at the end of
	 * the worksheet
	 * 
	 * @see isac.gui.treetable.CalculationModel
	 */	
	public Object[] updateInsertionNode() {
		if( insertionNode_ == null )
			insertionNode_ = new CalcModelNode((ICalcIterator)null);
			
		CalcModelNode lastNode = getLastNode( root_ );
		
		if( lastNode == null ) {
			System.out.println("In update insertion node : LastNode is null, bailing out");
			return null;
		}
		
		if( lastNode == insertionNode_ ) {
			System.out.println("In update insertion node : insertion node still exists, bailing out");
			return null;
		}
		
		if( lastNode.getPosition() == null )
			return addInsertionNode();
		
		Position lastPos = new Position();
		lastPos.setKind("Res");
		 
		if( lastNode.getPosition().compareTo( lastPos ) == 0 ) {
			//System.out.println("Last node encountered ...");
			return null;
		}
		
		if( lastNode != root_ )
			lastInsertionParent_ = lastNode.getParent();
		else
			lastInsertionParent_ = root_;
	
		int index2add = lastInsertionParent_.getChildCount();	
		lastInsertionParent_.add( index2add, insertionNode_ );
				
		if( lastInsertionParent_ == root_ )
			return new Object[] { new Object[] { root_ },
				new int[] { index2add - 1 }, new Object[] { insertionNode_ } };
		else
			return new Object[] { lastInsertionParent_.getPath().toArray(),
					new int[] { lastInsertionParent_.getChildIndex(insertionNode_) }, new Object[] { insertionNode_ } };	
	}

	/**
	 * @return Object[3] for treeEvents with an empty node the result has the
	 *         following structure: result[0] = path to the parent result[1] =
	 *         int[1] with the index of the removed Node as only element
	 *         result[2] = Object[1] with the removed Node as only element
	 * 
	 * this code was created for the removal of an empty node at the end of
	 * the worksheet
	 * 
	 * @see isac.gui.treetable.CalculationModel
	 */	
	public Object[] removeInsertionNode() {
		
		int index = lastInsertionParent_.getChildIndex( insertionNode_ ); 
		
		if( index < 0 )
			return null;
		
		lastInsertionParent_.remove( insertionNode_ );

		return new Object[] { lastInsertionParent_.getPath().toArray(),
				new int[] { index }, new Object[] { insertionNode_ } };		
	}
}