/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.treetable;

import isac.gui.IGuiConstants;
import isac.gui.Worksheet;
import isac.gui.mawen.syntax.Ast;
import isac.interfaces.IEditor;

import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.swing.plaf.ColorUIResource;
import javax.swing.tree.TreeCellRenderer;

/**
 * Custom Tree Cell Renderer
 * @version 1.0
 * @author Mario Hochreiter
 */
public class CalcTreeCellRenderer
  extends JPanel
  implements TreeCellRenderer {
	
  
	
  static final long serialVersionUID = 6590921149518486656L;
  
  
  
  protected Worksheet worksheet_;
  
  private JLabel image;
  private TreeTextPanel text;

  public CalcTreeCellRenderer( Worksheet ws ) {
    setLayout(new BorderLayout());
    image = new JLabel();
    
    
    text = new TreeTextPanel();
    
    add(image, BorderLayout.WEST);
    
    add(text, BorderLayout.CENTER);
    
    
    image.setOpaque(true);
    this.setOpaque(true);
    worksheet_ = ws;
  }

  public Component getTreeCellRendererComponent(
    JTree tree,
    Object value,
    boolean selected,
    boolean expanded,
    boolean leaf,
    int row,
    boolean hasFocus) {

    
    String stringValue =
        tree.convertValueToText(value, selected, expanded, leaf, row, hasFocus);
  	try {
  		text.setTacticText(((CalcModelNode)value).getSecondaryString() + "  ");
  	    
  	} catch (Exception e) {
  		text.setTacticText("");
  	}
  	
  	CalcModelNode current_node = (CalcModelNode) value;
  	
  	if (current_node.getFormula() == null) {
      // new Ast --> empty
/*err*/text.setFormula(new Ast.Constant(""));
    }else {
/*err*/text.setFormula(current_node.getFormula().getFormula().getTerm() );
    }
  	

  	text.setFormulaText(stringValue + "  ");
  	text.setSelect(selected);
    text.setFocus(hasFocus);
    
    
    if (selected) {
      setForeground(UIManager.getColor("Tree.selectionForeground"));
    } else
      setForeground(UIManager.getColor("Tree.Foreground"));
    // There needs to be a way to specify disabled icons.
    if (!tree.isEnabled()) {
      setEnabled(false);
      if (leaf) {
        setDisabledIcon(UIManager.getIcon("Tree.leafIcon"));
      } else if (expanded) {
        setDisabledIcon(UIManager.getIcon("Tree.openIcon"));
      } else {
        setDisabledIcon(UIManager.getIcon("Tree.closedIcon"));
      }
    } else {
      setEnabled(true);
      if (leaf) {
        setIcon(UIManager.getIcon("Tree.leafIcon"));
      } else if (expanded) {
        setIcon(UIManager.getIcon("Tree.openIcon"));
      } else {
        setIcon(UIManager.getIcon("Tree.closedIcon"));
      }
    }
    
    setComponentOrientation(tree.getComponentOrientation());
    //this.setBackground(UIManager.getColor("Table.Foreground"));
    
    
    this.setSize(new Dimension(getPreferredSize().width, text.getFormulaTextField().getExpressionSize().height));
    
    //this.revalidate();
    //tree.revalidate();

    text.revalidate();
    text.setSize(getPreferredSize());


    
    return this;
  }
  
  public Dimension getSize() {
		Dimension textD = text.getPreferredSize();
		int height = image.getSize().height < textD.height ? textD.height : image.getSize().height;
		Rectangle r = worksheet_.getBounds();
		int width = r.width == 0 ? IGuiConstants.WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X - 50 : r.width - 50;
		//return new Dimension(iconD.width + textD.width, height);
		return new Dimension(width, height);
	    
	  
  }
  
  public Dimension getPreferredSize() {
    return getSize();
  }
  
  public void setDisabledIcon(Icon icon) {
    image.setIcon(icon);   
  }

  public void setIcon(Icon icon) {
    image.setIcon(icon);    
  }
  
  public Dimension getFormulaSize() {
	  return text.getFormulaTextField().getExpressionSize();
  }
  
}

class TreeTextArea extends JTextArea {
  static final long serialVersionUID = 5536513715221108656L;

Dimension preferredSize;

  TreeTextArea() {
    setLineWrap(true);
    setWrapStyleWord(true);
    setOpaque(true);
  }

  public void setBackground(Color color) {
    if (color instanceof ColorUIResource)
      color = null;
    super.setBackground(color);
  }

  public void setSize(Dimension d) {
    if (d != null) {
      preferredSize = d;
    }
  }

  public Dimension getSize() {
    return preferredSize;
  }
  
  public Dimension getPreferredSize() {
    return preferredSize;
  }

  public void setText(String str) {
    BufferedReader br = new BufferedReader(new StringReader(str));
    FontMetrics fm = this.getFontMetrics(this.getFont());
    String line;
    int lines = 0;
    int maxWidth = 0;
    try {
      while ((line = br.readLine()) != null) {
        int width = SwingUtilities.computeStringWidth(fm, line);
        if (maxWidth < width) {
          maxWidth = width;
        }
        lines++;
      }
    } catch (IOException ex) {
      ex.printStackTrace();
    }
    lines = (lines < 1) ? 1 : lines;
    int height = fm.getHeight() * lines;
    setSize(new Dimension(maxWidth + 6, height));
    super.setText(str);
  }
}

class TreeTextPanel extends JPanel {
  static final long serialVersionUID = -195808786451707391L;

  private IEditor formulaText;
  private IEditor tacticText;

  TreeTextPanel() {
	   
    tacticText = EditorFactory.getTacticEditor();
    formulaText = EditorFactory.getFormulaEditor();
    
    
    
    this.setOpaque(true);
    this.setBackground(UIManager.getColor("Tree.background"));
    
    GridBagLayout gbl = new GridBagLayout();
    this.setLayout(gbl);
    
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
        
    gbc.anchor = GridBagConstraints.WEST;
    gbl.setConstraints( formulaText.getComponent(), gbc );
    this.add(formulaText.getComponent(), gbc);
    
    
    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.EAST;
    gbc.insets = new Insets( 0, 30, 0, 0 );
    gbl.setConstraints( tacticText.getComponent(), gbc );
    this.add(tacticText.getComponent(), gbc);
  }
  
  void setFormulaText(String s) {
    formulaText.setFormula(s);  
    this.repaint();
  }
  
  
  void setFormula(Ast.Ast ast) {
    formulaText.setFormula(ast);
    this.repaint();
  }
  
  void setTacticText(String s) {
    tacticText.setFormula(s);
    this.repaint();
  }
  
  IEditor getFormulaTextField() {
	  return formulaText;
  }
  
  IEditor getTacticTextField() {
	  return tacticText;
  }
  
  void setSelect(boolean isSelected) {
    Color bColor;
    if (isSelected) {
      bColor = UIManager.getColor("Tree.selectionBackground");
    } else {
      bColor = UIManager.getColor("Tree.textBackground");
    }
    super.setBackground(bColor);
  }
  
  public Dimension getPreferredSize() {
    
    int widthTactic = tacticText.getExpressionSize().width + 30;
    int widthFormula = formulaText.getExpressionSize().width;
    int heightTactic = tacticText.getExpressionSize().height;
    int heightFormula = formulaText.getExpressionSize().height;
    int newWidth = widthTactic > widthFormula ? widthTactic : widthFormula;
    return new Dimension(newWidth + 10, heightTactic + heightFormula);   
  }
  
  void setFocus(boolean hasFocus) {
    if (hasFocus) {
      Color lineColor = UIManager.getColor("Tree.selectionBorderColor");
      setBorder(BorderFactory.createLineBorder(lineColor));
    } else {
      setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
    }    
  }

} 
