/**
 * @author Alan Krempler
 * Created on Sep 11, 2003
 */
package isac.interfaces;

import java.io.Serializable;

/**
 * This is the interface common to all objects which can occur in a calculation
 * tree CalcTree.
 * 
 * @see isac.bridge.CalcTree
 * @author Alan Krempler
 */
public interface ICalcElement extends Serializable {

    /**
     * Labels a CalcElement as "Formula"
     * 
     * @see #getType()
     */
    public static final int CALCEL_FORMULA = 1;

    /**
     * Labels a CalcElement as "Tactic"
     * 
     * @see #getType()
     */
    public static final int CALCEL_TACTIC = 2;

    /**
     * Labels a CalcElement as "CalcHead"
     * 
     * @see #getType()
     */
    public static final int CALCEL_CALCHEAD = 3;

    /**
     * Labels a CalcElement as "Model"
     * 
     * @see #getType()
     */
    public static final int CALCEL_MODEL = 4;

    /**
     * Labels a CalcElement as "Assumption"
     * 
     * @see #getType()
     */
    public static final int CALCEL_ASSUMPTION = 5;

    //-----WN070720: review the above list looking as isac as is------------
    /**
     * This CalcElement will be displayed.
     * 
     * @see #getVisibility()
     */
    public static final int DISPLAY_VISIBLE_DEFAULT = 1;

    /**
     * This CalcElement will not be displayed, but it is possible to display it
     * on request.
     * 
     * @see #DISPLAY_INVISIBLE
     * @see #getVisibility()
     */
    public static final int DISPLAY_HIDDEN = 2;

    /**
     * This CalcElement will not be displayed, and will remain invisible.
     * 
     * @see #DISPLAY_HIDDEN
     * @see #getVisibility()
     */
    public static final int DISPLAY_INVISIBLE = 3;

    /**
     * This CalcElement will be displayed indicating there is something special
     * about it.
     * 
     * @see #getVisibility()
     */
    public static final int DISPLAY_VISIBLE_HIGHLIT = 4;

    /**
     * This CalcElement will be displayed but parts of it will be left blank to
     * be filled in by the user.
     * 
     * @deprecated This mechanism is likely to move to other parts of the system
     *             in the future.
     * @see #getVisibility()
     */
    public static final int DISPLAY_VISIBLE_FILL_IN = 5;

    /**
     * In the future, complicated elements like tactics might have view styles
     * of different complexity.
     * E.g. a formula might be shown with a minimum of parentheses or
     * with all parenthesis clarifying operator precedences --
     * -- TODO add respective constants.
     */
    public static final int VIEWSTYLE_DEFAULT = 1;

    /**
     * Every CalcElement is checked against its record in the UserModel and
     * rated as to how it will be treated by the WorksheetDialog. To avoid rating a
     * CalcElement at every contact over and over again, the results of the
     * rating are stored with the object. NOT_RATED expresses that this
     * CalcElement has not been rated yet.
     */
    public static final int NOT_RATED = 0;

    /**
     * Format the object in a way that the bridge can handle it
     * 
     * @return SML representation
     */
    public String toSMLString();

    /**
     * @return A MathML content representation of the data stored
     */
    public String getMathML();

    /**
     * @return An integer representing the type of the object
     * @see #CALCEL_FORMULA
     * @see #CALCEL_TACTIC
     * @see #CALCEL_CALCHEAD
     * @see #CALCEL_ASSUMPTION
     */
    public int getType();

    /**
     * @return An integer representing display options of the object
     * @see #DISPLAY_VISIBLE_DEFAULT
     * @see #DISPLAY_HIDDEN
     * @see #DISPLAY_INVISIBLE
     * @see #DISPLAY_VISIBLE_HIGHLIT
     * @see #DISPLAY_VISIBLE_FILL_IN
     */
    public int getVisibility();

    /**
     * @param visibility
     *            An integer representing display options of the object
     * @see #DISPLAY_VISIBLE_DEFAULT
     * @see #DISPLAY_HIDDEN
     * @see #DISPLAY_INVISIBLE
     * @see #DISPLAY_VISIBLE_HIGHLIT
     * @see #DISPLAY_VISIBLE_FILL_IN
     */
    public void setVisibility(int visibility);

    /**
     * @return An integer representing the view style of the object
     * @see #VIEWSTYLE_DEFAULT
     */
    public int getViewStyle();

    /**
     * @return An integer representing the view style of the object
     * @see #VIEWSTYLE_DEFAULT
     */
    public void setViewStyle(int view_style);

    /**
     * @return An integer representing the rating of the object
     * @see #NOT_RATED
     */
    public int getRating();

    /**
     * @return An integer representing the rating of the object
     * @see #NOT_RATED
     */
    public void setRating(int rating);
}