/*********************************************************************
 * This file is part of the Compod project
 * Contact: Alexander Nussbaumer (AN, alnuss)
 *   <alexander.nussbaumer@uni-graz.at>, <alnuss@gmx.net>
 *   
 *  modified: AN, 20070916
 *********************************************************************/

package isac.session;

/**
 * The interface between isac and Compod (learning path and assessment service)
 * 
 * The current version is subject to discussion with AN<br/>
 * 
 * For isac compod behaves the same during assessment and course.
 * Thus the respective methods are unified to *Session* in the identifers.
 * Only one session can be active per user.
 * isac's DialogGuide, however may distinguish assessment and course.
 *
 * Description of usage:<br/>
 * 1. setLMSID:
 * 		isac has to be registred externally and an ID is given from Compod.
 *    	This ID has to be used for every single communication step.
 *    	Therefore this ID is set via the setLmsID method, then the implementing
 *    	class will use this id for every further step. <br/>
 * 2. setExamples:
 *   	this is optional: sets the scope which the session is performed in
 * 3. startSesstion:
 * 		A session (assessment or course) is started. Can be done if no
 *   	other session is currently running. Usually an alternative sequence
 *   	of assessment and course should be conducted. <br/>
 * 4. getNextObjects:
 * 		an object list is retrieved from Compod <br/>
 * 5. notifyObjectMicroResult:
 * 		while student is working on object, the micro results (rechenregel)
 * 		are transmitted to Compod <br/>
 * 6. notifyObjectResult:
 * 		when an object has been finished, the result is sent to Compod.
 *      This is necessary during the assessment, but not necessary during
 *      an course. <br/>
 * 7. getSesstionStatus:
 * 		if true, then goto4. getNextObjects, otherwise finish <br/>
 * 
 * @author Alexander Nussbaumer (AN) 
 *    alexander.nussbaumer@uni-graz.at, alnuss@gmx.net
 *    
 * WN090109 calls from UserLogger as much as possible
 */
public interface ILMService {

	// TODO: use special exception for compod class/interface
	// TODO: consider of using enums for session (assessment and course)
	//   instead of string
	
	
	// --- enums ---
	
	/**
	 * Session State. Currently there are only two states, but perhaps 
	 * there will be more in the future.
	 *   ACTIVE: session (assessment or course) is active
	 *   INACTIVE: session (assessment or course) is not active
	 */
	public enum SessionState { ACTIVE, INACTIVE };
	
	/**
	 * Status of the knowledge and competence state.
	 *   AVAILABLE:  knowledge state is available when session inactive
	 *   UNAVAILABLE: knowledge state unavailable when state is active
	 *   PRILIMINARY: knowledge state preliminary when assessment session
	 *     has been stopped preliminary 
	 */
	public enum KnowledgeStateQuality { AVAILABLE, UNAVAILABLE, PRELIMINARY};
	
    /**
     * A session can be started as:
     *    ASSESMENT: The user might not be able to see all content, or might not be able to 
     *               use all functionality.
     *    COURSE: The user see's all content and has all user actions available. 
     */
    public enum SessionType { ASSESMENT, COURSE }
	
	// --- methods ---
	
	/**
     * (step 1) Sets the ID for communication between isac's LMS-service
     * and client. This ID is needed in every single communication step.
     * If it is not available, an exception will be thrown.
     * The ID is set manually in LMService.properties and by the SessionManager.
     * This method instantly tests the connection to the Web service.
     * 
     * @param lmsid
     * 	The LMD-ID which will used by the implementing class for further processing
     * @throws Exception
     * 	If no connection to the Web service could be established
     */
    public void setLMSID (String lmsid) throws Exception;

    /**
     * (step2) Sets the scope for the LMS session.
     *
     * @param username
     *   The user (unique name) for whom the action has to be performed.
     * @param topic
     *   The topic which the session has to be performed for.
     *   The topic string must match with the ontology in Compod.
     *   If the topic sting is null then the scope is reset to the default
     *   (full onology/scope).
     * @throws Exception
     *   If no connection to Compod Web service could be established
     */
    public void setScopeByTopic (String username, String topic) throws Exception;

    /**
     * (step2) Sets the examples to be calculated in the LMS session.
     *
     * @param username
     *   The user (unique name) for whom the action has to be performed.
     * @param topic
     *   The topic which the session has to be performed for.
     *   The topic string must match with the ontology in Compod.
     *   If the topic sting is null then the scope is reset to the default
     *   (full onology/scope).
     * @throws Exception
     *   If no connection to Compod Web service could be established
     */
    public void setExamples (String username, String... examples) throws Exception;

    /**
     * (step 3) Starts an session for an user. Can be done if the status is
     * "invalid" or "finished". Only one session can be active for an user
     * at the same time. Therefore no session ID is neccessary.
     * 
     * ###isac-call-location: session
     * 
     * @param username
     *  	The user (unique name) for whom the action has to be performed.
     * @param sessiontype
     * 		"assessment" xor "course" (### perhaps use enums)
     * @return 
     * 		success state
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public boolean startSession(String username, SessionType sessiontype) throws Exception;

    /**
     * (step 9) Stops a running session prematurely. Can be done if the
     * status is "running". It is not defined if a valid knowledge and
     * competence state can be achieved. see getKnowlegeState. 
     * 
     * @param username
     *            The user (unique name) for whom the action has to be
     *            performed.
     * @param cancel
     *            determines if the assessment is completely canceld (true), 
     *            or if the knowledge and competecen states are calculated 
     *            anyway (with low accuracy) (false)
     *            ?WN071010 SET ALWAYS false: better decided by Compod !?!
     * @return success state (currently not used)
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public boolean stopSession(String username, boolean cancel) throws Exception;

    /**
     * (step 7), iteration to (4) Returns the status of the current assessment.
     * Status is one of the SessionState
     * 
     * ###isac-call-location: exampleDialgo | Session
     * if "finished" zurueck zum bisherigen UserModel
     * 
     * @param username
     * 		The user (unique name) for whom the action has to be performed.
     * @return 
     * 		The session state
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public SessionState getSessionState(String username) throws Exception;

    /**
     * (step 4) Returns the next problems. The LMS should take the first one to
     * present it the user and ignore the others, or it can present all to the
     * user and the user decides which one is used.
     * depends on the status (getSessionStatus): if "inactive" no object is returned,
     * if "active" objects are returned, if "finnished" undefined behaviour (#####)
     * 
     * ###isac-call-location: exampleDialog
     * 
     * @param username
     *            The user (unique name) for whom the action has to be
     *            performed.
     * @return An array of object ids is returned. if no object is
     *         available, then a empty array is returned (String[0]).
     *         if no session is running, then null is returned.
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public String[] getNextObjects(String username) throws Exception;

    /**
     * (step 6) Accepts information if an isac object has solved
     * correctly.
     *
     * ###isac-call-location: WorksheetDialog | ExampleDialog for end of exp
     * 
     * @param username
     *            The user (unique name) for whom the action has to be
     *            performed.
     * @param objectid
     *            The object which the user is working on
     * @param solved
     *            solved or not
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public void notifyObjectResult(String username, String objectid, boolean solved) throws Exception;

    /**
     * (step 5) Accepts information which parts (skills) withn an assessment
     * objects has been solved to which extent (mastery level).
     * 
     * ###isac-call-location: ???
     *
     * @param username
     *            The user (unique name) for whom the action has to be
     *            performed.
     * @param objectid
     *            The object which the user is working on
     * @param skillid
     *            The skill which an user has solved (or not)
     * @param masterylevel
     *            The mastery level. ?WN070907? This is ...
     * @param solved
     *            solved or not
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public void notifyObjectMicroResult(String username, String objectid, 
    		String skillid, String masterylevel, boolean solved) throws Exception;


    // ============ get user data ===========================================
    // ?WN070907? when updated, when in a usable state, ...
    // ======================================================================

  
    /**
     * Returns information about the quality of the current knowledge
     * and competence state. 
     * "available": usable states are available (if assessment finnished normally)
     *   of at the beginning (then empty state)
     * "priliminary": priliminary state is available (if assessment was stopped)
     * "unavailable": no states are available (usually during assessment) 
     * 
     * @param username
     *      The user (unique name) for whom the action has to be performed.
     * @return The status string, one of [see above]
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public KnowledgeStateQuality getKnowledgeStateQuality(String username) throws Exception;


    /**
     * ?WN070907? - AN20070911: modified 
     * (step 8a) Returns the knowledge state of a user. The knowledge state is
     * updated internally during a session (assessment) and is valid
     * after a session has been finished. A valid session is indicated by the
     * method getKnowledgeStateQuality.
     * The state consists of a set of object IDs which the user is supposed to solve.
     * All other objects in the scope can not be solved by the user.
     * 
     * @param username
     * 		The user (unique name) for whom the action has to be performed.
     * @return The knowledge state (set of object IDs) String[0], if empty set.
     *         null, if not a valid state
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public String[] getKnowledgeState(String username) throws Exception;

    /**
     * ?WN070907? - AN20070911: modified 
     * (step 8a) Returns the competence state of a user. The competence state is
     * updated internally during a session (assessment or course) and is valid
     * after a session has been started or finnished. 
     * The state consist of a set of 'rechenregeln' IDs which the
     * user is supposed to be capable of All other 'rechenregeln' in the scope
     * can not be solved by the user. If the status is "running", the the state
     * is priliminary.
     * 
     * @param username
     *            The user (unique name) for whom the action has to be
     *            performed.
     * @return The competence state (set of skills) String[0], if empty set.
     *         null if not valid
     * @throws Exception
     * 		If no connection to Compod Web service could be established			
     */
    public String[] getCompetenceState(String username) throws Exception;

}
