/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.session;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

import isac.browserdialog.ProblemDialog;
import isac.browserdialog.MethodDialog;
import isac.browserdialog.TheoryDialog;
import isac.browserdialog.ExampleDialog;
import isac.browserdialog.BrowserDialog;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.WSDialogManager;
import isac.wsdialog.IContextProvider;
import isac.wsdialog.WorksheetDialog;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnBrowserDialog;
import isac.users.User;
import isac.users.UserLanguage;
import isac.users.UserManager;
import isac.util.formulae.KEStoreKey;
import isac.browserdialog.IContextPresenter;
import isac.interfaces.IToGUI;
import isac.users.LogItem;
import isac.users.LogItem.DialogTypes;
//UL import isac.users.UserLogger;

import org.apache.log4j.Logger;

/**
 * This class contains all browser dialogs and a worksheet dialog handler. It
 * is also used to get a IContextProvider, which is the interface for the
 * communication between the browser dialogs and the active worksheet. Sessions
 * can be accessed over the SessionManager.
 * 
 * @see isac.session.SessionManager
 * @author Robert Koenighofer
 */
public class Session extends UnicastRemoteObject implements ISession, Serializable {

    static final long serialVersionUID = 233953443;

    private static final Logger logger = Logger.getLogger(Session.class
            .getName());

    private String session_id_;

    private ProblemDialog problem_dialog_;

    private MethodDialog method_dialog_;

    private TheoryDialog theory_dialog_;

    private ExampleDialog example_dialog_;

    private WSDialogManager ws_dialog_manager_;

    private IToGUI interface_to_gui_;

    private BrowserDialog active_browser_dialog_;

    private HashMap<String, KEStoreKey> wsid_map_example_;

    protected UserLanguage user_language_;

    protected User user_;

//UL    protected UserLogger user_logger_;

    protected ILMService compod_;
    
    private LearningMode learning_mode_;
    
    private DialogGuide dialog_;

    /**
     * Constructor Each Session creates its own browser dialoges and a worksheet
     * dialog handler.
     * 
     * @param session_id
     *            the session id of the session to create
     * @throws RemoteException
     *             if remote object is not available
     */
    public Session(String session_id, User user, ILMService compod, DialogGuide dialog)
            throws RemoteException {
        session_id_ = session_id;
        user_ = user;
        user_language_ = user_.getLanguage();
        active_browser_dialog_ = null;
        dialog_ = dialog;

        // both must be before new *Dialog(this);
        compod_ = compod;
//UL        user_logger_ = new UserLogger(session_id, user_);
        LogItem log_item = new LogItem(user_.getUsername(), session_id_, "", 
          DialogTypes.NONE, "", "", null, "", EUIElement.LO_START_SESSION);
        dialog_.writeToDB(log_item);
        
        problem_dialog_ = new ProblemDialog(this, dialog);
        method_dialog_ = new MethodDialog(this, dialog);
        theory_dialog_ = new TheoryDialog(this, dialog);
        example_dialog_ = new ExampleDialog(this, dialog);
        ws_dialog_manager_ = new WSDialogManager(this, dialog);
        
        wsid_map_example_ = new HashMap<String, KEStoreKey>();

//UL        user_logger_.setExampleDialog(example_dialog_);
//UL        user_logger_.setSession(this);
    }

    /**
     * returns the Hierarchy Key of the example, which started the worksheet
     * dialog with the specified worksheet dialog id. If the dialog with the
     * specified ID was not started from an example, null is returned.
     * 
     * @param wsd_id
     *            the worksheet dialog id
     * @return the HierarchyKey of the example of the worksheetdialog or null
     */
    public KEStoreKey getExampleOfWorksheet(String wsd_id) {
        return wsid_map_example_.get(wsd_id);
    }

    /**
     * adds a new hierarchy key of a started example to the map. This is always
     * done, when a new worksheet dialog is started on a selecte example.
     * 
     * @param wsd_id
     *            the worksheet dialog id
     * @param example
     *            the HierarchyKey of the example which started the dialog
     */
    public void addWorksheetWithExample(String wsd_id, KEStoreKey example) {
        wsid_map_example_.put(wsd_id, example);
    }

    /**
     * removes an entry containing the worksheet dialog ID and the HierarchyKey
     * of the started example from the map. This method is called, whenever a
     * worksheet is closed. If the worksheet was not started from an example,
     * nothing happens.
     * 
     * @param wsd_id
     *            the worksheet dialog id
     */
    public void removeWorksheetWithExample(String wsd_id) {
        wsid_map_example_.remove(wsd_id);
    }

    /**
     * returns the session ID
     * 
     * @return the session ID
     * @throws RemoteException
     *             if remote object is not available
     */
    public String getId() throws RemoteException {
        return session_id_;
    }

    /**
     * returns the problem dialog of the session
     * 
     * @return the problem dialog of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public ProblemDialog getProblemDialog() throws RemoteException {
        return problem_dialog_;
    }

    /**
     * returns the method dialog of the session
     * 
     * @return the method dialog of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public MethodDialog getMethodDialog() throws RemoteException {
        return method_dialog_;
    }

    /**
     * returns the theory dialog of the session
     * 
     * @return the theory dialog of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public TheoryDialog getTheoryDialog() throws RemoteException {
        return theory_dialog_;
    }

    /**
     * returns the example dialog of the session
     * 
     * @return the example dialog of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public ExampleDialog getExampleDialog() throws RemoteException {
        return example_dialog_;
    }

    /**
     * returns the WSDialogManager of the session
     * 
     * @return the WSDialogManager of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public WSDialogManager getWSDialogManager() throws RemoteException {
        return ws_dialog_manager_;
    }

    /**
     * returns a IContextProvider, which acts as the interface between the the
     * browser dialoges and the active worksheet dialog. This interface can only
     * be used by the browser dialoges to access the active worksheet dailog.
     * The IContextProvider is implemented by the Worksheetdialog. The returned
     * interface belogs to the currently active WorksheetDialog. If no
     * WorksheetDialog is active at the moment (e.g. if there is no opened
     * WorksheetDialog at all), null is returned. The WSDialogManager knows the
     * active WorksheetDialog.
     * 
     * @return the IContextProvider or null if there is no active
     *         WorksheetDialog
     * @throws RemoteException
     *             if remote object is not available
     * @see isac.wsdialog.WSDialogManager#getActiveWSDialog()
     */
    public IContextProvider getContextProvider() throws RemoteException {
        return (IContextProvider) ws_dialog_manager_.getActiveWSDialog();
    }

    /**
     * returns a IContextPresenter, which acts as the interface between the the
     * browser dialoges and the active worksheet dialog. This interface can only
     * be used by the active worksheet dialoges to commnicate to the method
     * dialog. The IContextPresenter interface is implemented by all browser
     * dialoges. The returned interface belogs to the currently active browser
     * dialog. The active browser dialog is the one which is shown on top.
     * 
     * @return the IContextPresenter interface of the MethodDialog of this
     *         Session
     */
    public IContextPresenter getContextPresenter() {
        return (IContextPresenter) active_browser_dialog_;
    }

    /**
     * Sets a Browser Dialog the active browser dialog. This is done by the
     * browser dialoges themself whenever they get the focus.
     * 
     * @param new_active_dialog
     *            the browser dialog to set as active
     */
    public void setActiveBrowserDialog(BrowserDialog new_active_dialog) {
        active_browser_dialog_ = new_active_dialog;
    }

    /**
     * This method is called when one of the BrowserFrames is closed by the
     * user. Checks if the given BrowserDialogue is registered as active and if
     * that is the case, then the active dialogue is reset.
     * 
     * @param inactive_dialog
     */
    public void setInactiveBrowserDialog(BrowserDialog inactive_dialog) {
        if (active_browser_dialog_ == inactive_dialog)
            active_browser_dialog_ = null;
    }

    /**
     * returns an interface to the WindowApplication of the session
     * 
     * @return an interface to the window application of the session
     * @throws RemoteException
     *             if remote object is not available
     */
    public IToGUI getIToGUI() throws RemoteException {
        return interface_to_gui_;
    }

    /**
     * Initialises the WindowApplication. This method sends the buttons to the
     * window application and asks the WindowApplication to create the
     * BrowserFrames.
     * 
     */
    private void initialiseWindowApplication() throws RemoteException {
        if (interface_to_gui_ == null)
            return;

        if (logger.isDebugEnabled())
            logger.debug("----- add buttons to browsers--------------------");
        interface_to_gui_.addUIElement(new UIAction(
                EUIElement.UI_OPEN_WORKSHEET,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_));

        interface_to_gui_.addUIElement(new UIAction(
                EUIElement.UI_SHOW_EXAMPLE_BROWSER,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_));

        interface_to_gui_.addUIElement(new UIAction(
                EUIElement.UI_SHOW_THEORY_BROWSER,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_));

        interface_to_gui_.addUIElement(new UIAction(
                EUIElement.UI_SHOW_PROBLEM_BROWSER,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_));

        interface_to_gui_.addUIElement(new UIAction(
                EUIElement.UI_SHOW_METHOD_BROWSER,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_));

        if (logger.isDebugEnabled())
            logger.debug("----- add the browser frames--------------------");
        interface_to_gui_.doUIAction(new UIActionOnBrowserDialog(
                EUIElement.UI_OPEN_BROWSER_FRAME,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_,
                this.example_dialog_, "Example browser"));

        interface_to_gui_.doUIAction(new UIActionOnBrowserDialog(
                EUIElement.UI_OPEN_BROWSER_FRAME,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_,
                this.theory_dialog_, "Theory browser"));

        interface_to_gui_.doUIAction(new UIActionOnBrowserDialog(
                EUIElement.UI_OPEN_BROWSER_FRAME,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_,
                this.problem_dialog_, "Problem browser"));

        interface_to_gui_.doUIAction(new UIActionOnBrowserDialog(
                EUIElement.UI_OPEN_BROWSER_FRAME,
                EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_,
                this.method_dialog_, "Method browser"));
    }

    /**
     * Handles the requests from <code>WindowApplication</code>
     */
    public boolean notifyUserAction(IUserAction action) throws RemoteException,
            DialogProtocolException {
        System.out
                .println("In Session::notifyUserAction(IUserAction action) : Action is "
                        + action.getActionID()
                        + ", Context is "
                        + action.getContext());

        EUIElement actionID = action.getActionID();
        switch (actionID) {
        case UI_SHOW_EXAMPLE_BROWSER:
            example_dialog_.showBrowserFrame();
            break;
        case UI_SHOW_METHOD_BROWSER:
            method_dialog_.showBrowserFrame();
            break;
        case UI_SHOW_PROBLEM_BROWSER:
            problem_dialog_.showBrowserFrame();
            break;
        case UI_SHOW_THEORY_BROWSER:
            theory_dialog_.showBrowserFrame();
            break;
        case UI_OPEN_WORKSHEET:
            ws_dialog_manager_.openWSDialog();
            break;
        case UI_STOP_SESSION: 
            LogItem log_item = new LogItem(user_.getUsername(), session_id_, "", 
              DialogTypes.NONE, "", "", null, "", EUIElement.UI_STOP_SESSION);
            dialog_.writeToDB(log_item);
            
            UserManager.getInstance().logout(this);
            break;        
        default:
            WorksheetDialog wd = this.ws_dialog_manager_.getActiveWSDialog();
            if (wd != null) {
                wd.notifyUserAction(action);
            }
        }
        return true;
    }

    /**
     * Registers the WindowApplication to this session
     */
    public void registerBrowserFrame(IToGUI window_application)
            throws RemoteException {
        interface_to_gui_ = window_application;
        initialiseWindowApplication();
    }

    public UserLanguage getLanguage() {
        return user_language_;
    }

//UL    public UserLogger getUserLogger() {
//UL        return user_logger_;
//UL    }

    /**
     * <b>Do not use.</b><br></br>
     * This is "Compod garbage" and respective code will be removed.
     */
    public User getUser() {
        return user_;
    }

    /**
     * <b>Do not use.</b><br></br>
     * This is "Compod garbage" and respective code will be removed.
     */
    public KEStoreKey getExampleId(String ws_id) {
        return wsid_map_example_.get(ws_id);
    }

    public ILMService getCompod() {
        return compod_;
    }

    public LearningMode getLearningMode() {
        return learning_mode_;
    }

    public void setLearningMode(LearningMode learningMode) {
        this.learning_mode_ = learningMode;
    }

    /**
     * {@inheritDoc}
     */
    public boolean openWorksheetFromExample(String filename) throws RemoteException {
        return getExampleDialog().openWorksheetFromExample(filename + ".xml");
    }
}
