/*
 * @author Alan Krempler
 * Created on Dec 12, 2003
 */
package isac.useractions;

/**
 * Constants describing interactions or request from the user
 * 
 * WN0710 these are _all_ user interactions possible in isac. They are used to
 * notify the respective dialogs; however, they include some technicalities like
 * UI_DUMMY_LAST_SOLVE. The interactions are logged fairly 1:1; however, this
 * requires some additional actions created only for logging; these begin with
 * LO_
 */
public enum EUIElement {

    /** ***************** NOTIFICATIONS: SPECIFYING PHASE ********************** */

    UI_DUMMY_FIRST_SPECIFY,

    /**
     * During the specifying phase, change the view on the CalcHead. WN050804
     * how to use ? which view ?
     */
    UI_SPECIFY_CHANGE_VIEW,

    /**
     * During the specifying phase, have the Model of the problem (or the guard
     * of the method) completed by the math engine.
     */
    UI_SPECIFY_COMPLETE_MODEL,

    /**
     * During the specifying phase, have the CalcHead completed by the math
     * engine.
     */
    UI_SPECIFY_COMPLETE_CALCHEAD,

    /**
     * During the specifying phase, have the Method field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_METHOD,

    /**
     * During the specifying phase, have the Theory field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_THEORY,

    /**
     * During the specifying phase, have the Problem field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_PROBLEM,

    /**
     * During the specifying phase, have the Given field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_COMPLETE_GIVEN,

    /**
     * During the specifying phase, have the Find field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_COMPLETE_FIND,

    /**
     * During the specifying phase, have the Relate field of the CalcHead
     * completed by the math engine.
     */
    UI_SPECIFY_COMPLETE_RELATE,

    /**
     * During the specifying phase, have the CalcHead checked for completeness
     * and consistency.
     */
    UI_SPECIFY_CHECK_CALCHEAD,

    /**
     * Leave the specifying phase, enter the solving phase and calculate until a
     * final result is reached.
     */
    UI_SPECIFY_CALCULATE_ALL,

    /**
     * clear the (possibly messed) contents of the CalcHeadPanel such that a
     * UI_SPECIFY_COMPLETE_CALCHEAD or a UI_SPECIFY_CALCULATE_ALL can succeed.
     */
    UI_SPECIFY_RESET,

    /**
     * open the CalcHeadPanel and start the specify-phase, if the CalcHead is
     * not 'complete'
     */
    UI_SPECIFY_OPEN_CALCHEAD,

    /**
     * Leave the specifying phase, enter the solving phase and calculate the
     * first step of the method. A UserAction only.
     * 
     * @see UI_SPECIFY_CLOSE_CALCHEAD the closely related UIAction
     */
    UI_SPECIFY_TO_SOLVE,

    /**
     * close the CalcHeadPanel (i.e. a UIAction only)
     * 
     * @see UI_SPECIFY_TO_SOLVE the closely related UserAction
     */
    UI_SPECIFY_CLOSE_CALCHEAD,

    /**
     * fill calchead elementwise
     */
    UI_SPECIFY_COMPLETE_STEPWISE,

    /**
     * Show the guard of the method (already specified in the CalcHead). The
     * guard of the method has the same structure as the Model of a problem;
     * thus the guard is inserted into the CalcHead at the place of the problems
     * Model.
     * 
     * @see UI_SPECIFY_SWITCH_TO_MODEL switch to the other part of a toggle-mode.
     */
    UI_SPECIFY_SWITCH_TO_GUARD,

    /**
     * Switch back from having the guard of a method to having the Model of the
     * problem (already specified in the CalcHead).
     * 
     * @see UI_SPECIFY_SWITCH_TO_GUARD switch to the other part of a
     *      toggle-mode.
     */
    UI_SPECIFY_SWITCH_TO_MODEL,

    UI_SPECIFY_EDIT_ACTIVE_FORMULA_COMPLETE,

    /**
     * WN050830: better use UI_SPECIFY_SWITCH_TO_MODEL | GUARD ?!?
     * Or is there a problem with actionPerformed on radiobutton ?!?
     * in this case WN will implement ICalcIterator#onModel, #onGuard
     * 
     * @see UI_SPECIFY_SWITCH_TO_MODEL
     */
    UI_SPECIFY_TOGGLE_METHOD_PROBLEM,

    UI_DUMMY_LAST_SPECIFY,

    /** ***************** NOTIFICATIONS: SOLVING PHASE ********************** */

    UI_DUMMY_FIRST_SOLVE,

    /**
     * During the solving phase, calculate one more step.
     */
    UI_SOLVE_CALCULATE_1,

    /**
     * During the solving phase, calculate until a final result is reached.
     */
    UI_SOLVE_CALCULATE_ALL,

    /**
     * During the solving phase, calculate until the current subproblem is
     * solved.
     */
    UI_SOLVE_CALCULATE_SUBPROBLEM,

    /**
     * During the solving phase, request editing the currently active formula.
     * 
     * WN0412 indicates the request to send this formula to the MathEngine.
     * 
     * @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE
     */
    UI_SOLVE_EDIT_ACTIVE_FORMULA,

    /**
     * During the solving phase, notify that editing the currently active
     * formula is finished. This implies a request for updating the CalcTree.
     */
    UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,

    /**
     * During the solving phase, request inserting a new formula _after_ the
     * currently active formula, making the new formula the currently active
     * formula. At present, this does not imply editing the formula and must be
     * requested in a separate step. WN0412: ...??? is this right interpreted by
     * ... WN0412 If request is conceded, do
     * 
     * @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE
     */
    UI_SOLVE_APPEND_USER_FORMULA,

    /**
     * During the solving phase, request making the referened formula the
     * currently active formula.
     */
    UI_SOLVE_MOVE_ACTIVE_FORMULA,

    /**
     * During the solving phase, ask for the tactic the math engine would apply
     * to the current formula.
     */
    UI_SOLVE_GET_PROPOSED_TACTIC,

    /**
     * During the solving phase, ask for a list of tactics to the active
     * formula.
     * WN120730 the same as @see UI_SOLVE_SHOW_APPLICABLE_TACTICS ?
     */
    UI_SOLVE_GET_APPLICABLE_TACTICS,

    /**
     * During the solving phase, set the tactic to be applied to the current
     * formula.
     */
    UI_SOLVE_SET_NEXT_TACTIC,

    /**
     * When entering a formula during the solving phase, request help on
     * entering the formula.
     */
    UI_SOLVE_HELP_ENTERING_FORMULA,

    /**
     * During the solving phase, show which assumptions hold at the current
     * point of calculation.
     */
    UI_SOLVE_SHOW_ASSUMPTIONS,

    /**
     * Show which tactic has been applied to get this formula in specific kinds
     * of user-guidance
     * 
     * ... WN070726 usually there are more than 1 tactic in a sqequence to
     * justify an input formula
     */
    UI_SOLVE_TACTIC_APPLIED,

    /**
     * Show which tactics are applicable for this formula
     * WN120730 the same as @see UI_SOLVE_GET_APPLICABLE_TACTICS ?
     */
    UI_SOLVE_SHOW_APPLICABLE_TACTICS,

    /**
     * show CalcHeadPanel; if CalcHead "complete" no interactiveElements if not,
     * startSpecifying
     */
    UI_SOLVE_SHOW_CALCHEAD,

    /**
     * During the solving phase, request more detail about the currently
     * referenced element.
     */
    UI_SOLVE_SHOW_INTERMEDIATE_STEPS,

    /*switch to tree-representation for the active formula*/
    UI_SOLVE_TERM_TREE,
    
    /**
     * Change the currently selected formula. When the user clicks on the
     * formula in the worksheet, then the WorksheetDialog is notified. When the
     * calcChanged arrives, then the Worksheet gets notified.
     *
     * WN120730 replace by @see UI_SOLVE_MOVE_ACTIVE_FORMULA
     */
    UI_SOLVE_CHANGE_CONTEXT_FORMULA,

    /**
     * if no complete calculation is on this worksheet, this action indicates
     * failure in calculating the result.
     * 
     * @see isac.useractions.EUIElement.LO_OPEN_WORKSHEET
     */
    UI_CLOSE_WORKSHEET,

    /**
     * close the WindowApplication
     * 
     * @see isac.useractions.EUIElement.LO_START_SESSION
     */
    UI_STOP_SESSION,

    /**
     * This action is sent from the WS->WSDialog when the worksheet becomes
     * active
     */
    UI_ACTIVE_WORKSHEET,

    UI_DUMMY_LAST_SOLVE,

    /** ***************** ACTIVE REQUESTS TO THE UI ********************** */

    /**
     * Request editing the currently active formula. DG distinguishes between
     * SPECIFY and SOLVE; thus could drop UI_SOLVE_EDIT_ACTIVE_FORMULA : int
     * 
     * LK050711 start editing implicit somewhere in tree, thus unused
     */
    UI_DO_EDIT_FORMULA,

    /**
     * Request appending a new, empty formula.
     */
    UI_DO_APPEND_FORMULA,

    /**
     * Activate an UI element.
     */
    UI_DO_ACTIVATE,

    /**
     * Deactivate an UI element.
     */
    UI_DO_DEACTIVATE,

    /**
     * The Dialog Guide has detached the listener and will not send any more
     * control requests. If registered as a Data Change Listener, update
     * notifications will still arrive!
     */
    UI_DO_DETACH,

    /**
     * Transfer the problem from the problem browser to the worksheet
     */
    UI_PROBLEMBROWSER_TO_WORKSHEET,

    /**
     * Transfer the problem from the browser to the worksheet
     */
    UI_BROWSER_TO_WORKSHEET,

    /**
     * to turn the auto match mode on
     */
    UI_PROBLEMBROWSER_AUTO_MATCH_ON,

    /**
     * to turn the match mode on
     */
    UI_BROWSER_MATCH_ON,

    /**
     * to turn the auto match mode off
     */
    UI_PROBLEMBROWSER_AUTO_MATCH_OFF,

    /**
     * to turn the match mode off
     */
    UI_BROWSER_MATCH_OFF,

    /**
     * to open the problem browser window
     */
    UI_PROBLEMBROWSER_OPEN,

    /**
     * to tell the proble browser dialog that the selection in the problem
     * browser has changed
     */
    UI_PROBLEMBROWSER_SELECTION_CHANGED,

    /**
     * search for a more appropriate problem
     */
    UI_PROBLEMBROWSER_REFINE,

    /**
     * no 'UserAction', but but included here for simplicity reasons: to send a
     * request fot the selected problem
     */
    UI_PROBLEMBROWSER_SELECTED_ID,

    /**
     * no 'UserAction', but but included here for simplicity reasons: to send a
     * request fot the active worksheet
     */
    UI_PROBLEMBROWSER_ACTIVE_WORKSHEET,

    /**
     * to send a new model panel to the browser
     */
    UI_PROBLEMBROWSER_MODEL_PANEL,

    /**
     * To send a request for the hierarchy
     */
    UI_REQUEST_FOR_HIERARCHY,

    /**
     * The browser reports a context change, or receives a context change
     */
    UI_BROWSER_CHANGE_CONTEXT,

    /**
     * The browser requires a context reaction (the example in ExmapleBrowser is
     * double-clicked).
     */
    UI_BROWSER_EXECUTE_CONTEXT,

    /**
     * MiniBrowser reports/gets a link
     */
    UI_MINIBROWSER_LINK,

    /**
     * to set the focus on the browser window
     */
    UI_BROWSER_GET_FOCUS,

    /**
     * to focus was set on browser window
     */
    UI_BROWSER_GOT_FOCUS,

    /**
     * the focus in the borwser window got lost
     */
    UI_BROWSER_LOST_FOCUS,

    /**
     * 
     * initial action to transfer the hierarchy from BrowserDialog->Browser
     * 
     * note: ONLY a UIAction not a UserAction
     * 
     */
    UI_BROWSER_HIERARCHY,

    /**
     * asks the WindowApplication to open a new Worksheet
     */
    UI_OPEN_WORKSHEET,

    /**
     * asks the WindowApplication to open a new BrowserFrame
     */
    UI_OPEN_BROWSER_FRAME,

    /**
     * The WindowApplication asks the Session to display the ProblemBrowser
     */
    UI_SHOW_PROBLEM_BROWSER,

    /**
     * The WindowApplication asks the Session to display the MethodBrowser
     */
    UI_SHOW_METHOD_BROWSER,

    /**
     * The WindowApplication asks the Session to display the TheoryBrowser
     */
    UI_SHOW_THEORY_BROWSER,

    /**
     * The WindowApplication asks the Session to display the ExampleBrowser
     */
    UI_SHOW_EXAMPLE_BROWSER,

    /** ***************** ACTIONS ONLY REQUIRED FOR USERLOGGER ***************** */

    LO_START_SESSION, // UI_STOP_SESSION,

    /**
     * This includes START_EXAMPLE, while LO_EXAMPLE_FINISHED_SUCCESSFUL and UI_CLOSE_WORKSHEET
     * are separated.
     */
    LO_OPEN_WORKSHEET, // UI_CLOSE_WORKSHEET

    /**
     * this indicates _successful_ completion of a calcution, because there are
     * no errors allowed in a calculation. Closing the worksheet can be done
     * later.
     */
    LO_EXAMPLE_FINISHED_SUCCESSFUL

}