/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.users;

import isac.session.LearningMode;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

import org.apache.log4j.Logger;

import isac.session.SessionManager;
import isac.session.Session;

import isac.util.ObjectManagerPaths;

/**
 * This class administrates all users. It is implemented as Singleton
 * (design Pattern of Gamma et.al.), which means, that there can be only
 * one instance of this class. This instance can be accessed by the
 * getInstance() method.
 * 
 * @author Robert Koenighofer
 */
public class UserManager extends UnicastRemoteObject implements IUserManager, Serializable {

    static final long serialVersionUID = 67389427;
    private static UserManager instance;
    private HashMap<String, User> allLoggedInUsers;
    private static final Logger LOG = Logger.getLogger(UserManager.class.getName());

    /**
     * The constructor is private, since this class is implemented
     * as singleton. Use the getInstance() method to get the one and
     * only instance of this class.
     * 
     * @throws RemoteException if remote object is not available any more
     */
    private UserManager() throws RemoteException {
        allLoggedInUsers = new HashMap<String, User>();
    }

    /**
     * Checks by use of {@link isac.users.Accounts} and {@link isac.users.Domains} if username and
     * password is correct. If this is true a new Session is created by
     * use of the {@link isac.session.SessionManager}. The
     * created Session object is returned. If the login fails, null
     * is returned.
     * 
     * @param username the name of the user who wants to login
     * @param password the password the user has entered
     * @param learningMode 'Assessment' or 'learning'
     * @return the Session which was created for the user or null if login fails
     * @throws RemoteException if remote object is not available any more
     */
    public Session login(String username, String password, LearningMode learningMode) 
        throws RemoteException {
        Session newSession = null;
        String userpath = ObjectManagerPaths.SESSION_MANAGER_USER_PATH;
        User user = new User(username, userpath);
        if (authenticationValid(user, password)) {
            allLoggedInUsers.put(username, user);
            newSession = SessionManager.getInstance().addSession(user);
            newSession.setLearningMode(learningMode);
//            newSession.getIToGUI().addUIElement(new )
            
            LOG.info("WA<-SM: login <- sessionid=" + newSession.getId());
        } else {
            LOG.info("Username ('" + username + "'), or password ('" + password + "') wrong ");
        }

        return newSession;
    }

    /**
     * Return true if the authentication is valid, false otherwise.
     */
    public boolean isAuthenticationValid(String username, String password) throws RemoteException {
        User user = new User(username, ObjectManagerPaths.SESSION_MANAGER_USER_PATH);
        return authenticationValid(user, password);
    }
    
    private boolean authenticationValid(User user, String password) {
        if (user.hasDomain()) {
            Domains domains = new Domains(user.getUserpath());
            return domains.authenticationValid(user.getDomain(), password);
        } else {
            Accounts accounts = new Accounts(user.getUserpath());
            return accounts.authenticationValid(user.getUsername(), password);
        }
    }

    /**
     * Logs the user out.
     * This method removes the session from all maps to allow the garbage 
     * collector to destroy the object.
     * 
     * @param session the session to log out
     * @throws RemoteException if remote object is not available any more
     */
    public void logout(Session session) throws RemoteException {
        String sessionId = session.getId();
        SessionManager.getInstance().removeSession(sessionId);
    }

    /**
     * Returns the User object of the user belonging to the session
     * with the specified ID. If no Session with this ID is
     * known, null is returned.
     * 
     * @param session_id
     * @return the User who owns the session or null if session_id is unknown
     * @throws RemoteException if remote object is not available any more
     */
    public User getUser(String session_id) throws RemoteException {
        String username = SessionManager.getInstance().getUserNameByID(session_id);
        if (username == null) {
            return null;
        }
        return allLoggedInUsers.get(username);
    }

    /**
     * Returns the username of the user belonging to the session
     * with the specified ID as String. If no Session with this ID is
     * known, null is returned.
     * 
     * @param session_id
     * @return the username who owns the session or null if session_id is unknown
     * @throws RemoteException if remote object is not available any more
     */
    public String getUsername(String session_id) throws RemoteException {
        return SessionManager.getInstance().getUserNameByID(session_id);
    }

    /**
     * returns the instance of the singleton UserManager. Since the
     * constructor is private, this is the only method to get a
     * UserManager. It always returns one and the same instance of
     * this class.
     * 
     * @return the one and only instance of this class.
     * @throws RemoteException if remote object is not available any more
     */
    public static UserManager getInstance() throws RemoteException {
        if (instance == null) {
            try {
                instance = new UserManager();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
        return instance;
    }

       /**
     * For use in tests only !
     */
    public void PRIVATE_TEST_resetForTests (User user) {
      allLoggedInUsers.clear();
      allLoggedInUsers.put(user.getUsername(), user);
    }

}
