/*
 * Created on Feb 22, 2005
 *
 * @author Alan Krempler
 */
package isac.users;

import isac.util.FileUtils;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

/**
 * Class for storing/retrieving user settings, i.e. settings which can be set
 * manually by the user as opposed to the
 * {@link isac.util.usersettings.UserModel#}, which is an abstraction of the
 * system's expeience with a user and canot be altered manually. As there is no
 * fixed set of user settings, you can regard this class as a simple storage
 * backend for key/value pairs.
 * 
 * @author Alan Krempler
 *  
 */
public class UserSettings extends Properties implements IUserSettings {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Map<String, Integer> storage = new HashMap<String, Integer>();
    String userpath;
    String username;
    public static HashMap<String, Integer> valstr_valint_;
    private static final String USERSETTINGS_FILESUFFIX = "_settings.txt";
    private static final Map<String, Integer> USER_SETTINGS_MAPPING;
    private static final Logger LOG = Logger.getLogger(UserSettings.class.getName());

    static {
        // @see isac.uti.users.IUserSettings
        USER_SETTINGS_MAPPING = new HashMap<String, Integer>();
        USER_SETTINGS_MAPPING.put("DUMMY", new Integer(IUserSettings.DUMMY));
        USER_SETTINGS_MAPPING.put("VAL1_SKIP_SPECIFY_TO_START_SOLVE", IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE);
        USER_SETTINGS_MAPPING.put("VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS", IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS);
        USER_SETTINGS_MAPPING.put("VAL2_FORMULAE_ONLY", IUserSettings.VAL2_FORMULAE_ONLY);
        USER_SETTINGS_MAPPING.put("VAL2_FORMULA_TACTIC_FORMULA_", IUserSettings.VAL2_FORMULA_TACTIC_FORMULA_);
        USER_SETTINGS_MAPPING.put("VAL3_LEARNER", IUserSettings.VAL3_LEARNER);
        USER_SETTINGS_MAPPING.put("VAL3_AUTHOR", IUserSettings.VAL3_AUTHOR);
        USER_SETTINGS_MAPPING.put("VAL4_ENGLISH", IUserSettings.VAL4_ENGLISH);
        USER_SETTINGS_MAPPING.put("VAL4_DEUTSCH", IUserSettings.VAL4_DEUTSCH);
    }
    
    private static Properties DUMMY_USERSETTINGS = new Properties();

    static {
        DUMMY_USERSETTINGS.setProperty("KEY1_START_SPECIFY", "VAL1_SKIP_SPECIFY_TO_START_SOLVE");
        DUMMY_USERSETTINGS.setProperty("KEY2_NEXT_BUTTON", "VAL2_FORMULAE_ONLY");
        DUMMY_USERSETTINGS.setProperty("KEY3_STATUS", "VAL3_LEARNER");
        DUMMY_USERSETTINGS.setProperty("KEY4_LANGUAGE", "VAL4_ENGLISH");
    }    

    /**
     * 
     * @see isac.uti.users.IUserSettings
     */
    public UserSettings(String directoryOfUserModelFile, String username) {
        this.userpath = directoryOfUserModelFile;
        this.username = username;
        final String userSettingsFilePath = directoryOfUserModelFile + getUserSettingsFileName(username);

        createUserSettingsFileIfItDoesNotExist(userSettingsFilePath, username);
        
        Properties userSettingsFromFile = new Properties();
        loadUserModelFromFile(userSettingsFilePath, userSettingsFromFile);
        
        mapUserSettings(userSettingsFromFile, userSettingsFilePath);
    }

    private String getUserSettingsFileName(String username) {
        return username + USERSETTINGS_FILESUFFIX;
    }

    private void loadUserModelFromFile(String file, Properties properties) {
        FileUtils.loadPropertiesFromFile(file, properties, "user settings");
    }

    private static void createUserSettingsFileIfItDoesNotExist(String accountsFile, String username) {
        FileUtils.createPropertiesFileIfItDoesNotExist(accountsFile, DUMMY_USERSETTINGS, "user settings",
            "Static dialog data of user '" + username + "'."
            + "\nPossible values for the KEY1_xxx - KEY4_xxx = " + USER_SETTINGS_MAPPING.keySet().toString() 
            + "\nFollowing values are DUMMY values.");
    }
    
    /**
     * Use {@code userSettingsFromFile} and the mapping from {@code USER_SETTINGS_MAPPING} to store the result into
     * {@code storage_}
     * @param userSettingsFromFile 
     */
    private void mapUserSettings(Properties userSettingsFromFile, String userSettingsFilePath) {
        this.storage.clear();
        Enumeration<?> keysFromFile = userSettingsFromFile.propertyNames();//WN050311 how
        String keyFromFile = null;
        String valueFromFile = null;
        Integer mappedValue = null;
        while (keysFromFile.hasMoreElements()) {
            keyFromFile = (String) keysFromFile.nextElement();
            valueFromFile = userSettingsFromFile.getProperty(keyFromFile);
            mappedValue = USER_SETTINGS_MAPPING.get(valueFromFile);
            if (mappedValue == null) {
                LOG.severe("Could not find mapping for user setting value '" + valueFromFile + "' (key= '"
                    + keyFromFile + "') in user settings file '" + userSettingsFilePath + "'");
                LOG.severe("Available mappings for the value are: " + USER_SETTINGS_MAPPING.toString());
            }
            this.storage.put(keyFromFile, mappedValue);
        }
    }

    /**
     * @param key
     *            A string key
     * @return The string value associated with the key parameter; null, if
     *         nothing is stored under this key
     */
    public int getValue(String key) {
        return ((Integer) this.storage.get(key)).intValue();
    }

    /**
     * @param key
     *            The key under which "value" is to be stored. Existing data
     *            unde the same key will be replaced.
     * @param value
     *            The value to be stored under "key"
     * @return true in case of success, false in case of error, for example if
     *         trying to replace stored values without proper permissions
     */
    public boolean setValue(String key, int value) {
        //TODO.WN050311 storage_.put(key, value);
        return true;
    }
}
