package isac.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Logger;

/**
 *
 * @author tzilling@sbox.tugraz.at
 */
public class FileUtils {

    private static final Logger LOG = Logger.getLogger(FileUtils.class.getName());

    /**
     * Loads the contents of a properties File with the path and name {@code propertiesFile} into {@code properties}. 
     * {@code whatIsThisFile} is used for the logger. If an error occurs
     * this text is used to refer to the file. E.g. if {@code whatIsThisFile}= 'user accounts' then a logger message 
     * could be "Listing user accounts configured in c:/mydir/users/accounts.txt"
     */
    public static void loadPropertiesFromFile(String propertiesFile, Properties properties, String whatIsThisFile)
        throws RuntimeException {
        FileInputStream propertiesFileStream = null;
        try {
            propertiesFileStream = new FileInputStream(propertiesFile);
            properties.load(propertiesFileStream);
        //System.out.println("Listing " + whatIsThisFile + " configured in '" + propertiesFile + "':");
        //properties.list(System.out);
        } catch (FileNotFoundException e) {
            LOG.severe("Cannot find " + whatIsThisFile + " file " + propertiesFile);
            System.err.println("Cannot find " + whatIsThisFile + " file " + propertiesFile);
        } catch (IOException e) {
            LOG.severe("Cannot load " + whatIsThisFile + " from file " + propertiesFile);
            System.err.println("Cannot load " + whatIsThisFile + " fromfile " + propertiesFile);
        } finally {
            try {
                if (propertiesFileStream != null) {
                    propertiesFileStream.close();
                }
            } catch (IOException ex) {
                throw new RuntimeException("Could not close FileStream", ex);
            }
        }
    }

    /**
     * Creates a properties File with the path and name {@code filePath} and filled with the initial
     * properties from {@code initialProperties}. {@code whatIsThisFile} is used for the logger. If an error occurs
     * this text is used to refer to the file. E.g. if {@code whatIsThisFile}= 'user accounts' then a logger message 
     * could be "Listing user accounts configured in c:/mydir/users/accounts.txt". 
     * @param additionalComment A comment that will be put at the beginning of the properties file.
     */
    public static void createPropertiesFileIfItDoesNotExist(String propertiesFile, Properties initialProperties,
        String whatIsThisFile, String additionalComment) {

        File porpertiesFile = new File(propertiesFile);

        if (!porpertiesFile.exists()) {
            LOG.warning(whatIsThisFile + " File '" + porpertiesFile + "' does not exist, creating new one with dummy "
                + whatIsThisFile + " properties");

            FileOutputStream propertiesFileStream = null;
            try {
                propertiesFileStream = new FileOutputStream(porpertiesFile);
                initialProperties.store(propertiesFileStream, "Automatically created list of " + whatIsThisFile
                    + " properties. " + StringUtils.emptyStringIfNull(additionalComment));
            } catch (IOException e) {
                LOG.severe("Could not write new " + whatIsThisFile + " file '" + porpertiesFile + "'.");
                LOG.severe(e.getMessage());
            } finally {
                try {
                    if (propertiesFileStream != null) {
                        propertiesFileStream.close();
                    }
                } catch (IOException ex) {
                    throw new RuntimeException("Could not close FileStream", ex);
                }
            }
        }
    }
}
