/*
 * @author Georg Kompacher, member of the ISAC-team, 
 * Copyright (c) 2006 by Georg Kompacher
 * created 18.07.2006 16:31:02
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.parser;

import isac.util.formulae.HierarchyKey;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;

import org.apache.commons.digester.Digester;
import org.apache.log4j.Logger;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class HierarchyKeyParser {

    private Digester digester_;
    private static final Logger logger = Logger.getLogger(HierarchyKeyParser.class.getName());

    // auxillary class for feeding the dtd_
    class KEStoreEntityResolver implements EntityResolver {

        String dtd_;

        public KEStoreEntityResolver(String dtd) {
            this.dtd_ = dtd;
        }

        public InputSource resolveEntity(String publicId, String systemId)
                throws SAXException, IOException {
            if ((systemId != null) && systemId.endsWith("ke_store.dtd")) {
                return new InputSource(new StringReader(dtd_));
            } else {
                System.out.println("KEStoreEntityResolver: return null");
                return null;
            }
        }
    }

    /**
     * Constructor: create a new HierarchyKeyParser object and fill it with the
     * rules.
     * 
     * @param dtd_path_
     *            Path to the DTD-File (Document Type Description)
     */
    public HierarchyKeyParser(String dtd_path)  {
        if (logger.isDebugEnabled()) {
            logger.debug("HierarchyKeyParser: dtd_path_=" + dtd_path);
        }

        digester_ = new Digester();

        StringBuffer sb;

        try(BufferedReader br = new BufferedReader(new FileReader(dtd_path));){
           
            sb = new StringBuffer();
            while (br.ready()) {
                sb.append(br.readLine() + "\n");
            }
            // Set dtd_ for parser validation
            digester_.setEntityResolver(new KEStoreEntityResolver(sb.toString()));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        // ============== Rules for parsing HierarchyKeys
        digester_.addObjectCreate("*/STRINGLIST", HierarchyKey.class);
        digester_.addCallMethod("*/STRINGLIST/STRING", "addString", 0);
    }

    public void setValidating(boolean validating) {
        if (logger.isDebugEnabled()) {
            logger.debug("setValidating: validating=" + validating);
        }
        digester_.setValidating(validating);
    }

    /**
     * Parse an XML string to Java Object(s)
     * 
     * @param xmlString
     *            the string containing the XML content
     * @return ResponseWrapper containing the CalcID and the parsed Object
     */
    public HierarchyKey parse(String xmlString) {
        Object o = null;

        encodeEntities(xmlString);
        System.out.println("xmlString is: " + xmlString);
        try {
            digester_.clear();
            o = digester_.parse(new InputSource(new StringReader(xmlString)));
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } catch (SAXException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        HierarchyKey hierarchy_key = (HierarchyKey) o;
        return hierarchy_key;
    }

    public static String encodeEntities(String xmlString) {
        xmlString = xmlString.replaceAll("&\n", " &amp; ");
        xmlString = xmlString.replaceAll(" & ", " &amp; ");
        xmlString = xmlString.replaceAll(" > ", " &gt; ");
        xmlString = xmlString.replaceAll(" < ", " &lt; ");
        xmlString = xmlString.replaceAll(" <= ", " &lt;= ");
        xmlString = xmlString.replaceAll(" >= ", " &gt;= ");
        xmlString = xmlString.replaceAll("&Ouml;", "?");
        xmlString = xmlString.replaceAll("&ouml;", "?");
        xmlString = xmlString.replaceAll("&Uuml;", "?");
        xmlString = xmlString.replaceAll("&uuml;", "?");
        xmlString = xmlString.replaceAll("&Auml;", "?");
        xmlString = xmlString.replaceAll("&auml;", "?");
        xmlString = xmlString.replaceAll("&nbsp;", " ");

        return xmlString;
    }
}
