/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.wsdialog;

import isac.util.formulae.Context;

/**
 * This is an interface for the browser dialoges to get the context
 * information about the active worksheet. It is implemented by the
 * WorksheetDialog. The browser dialoges ask their Session to deliver
 * the IContextProvider. The object behind this interface is always
 * the active WorksheetDialog of the Session.
 * 
 * @see isac.session.Session#getContextProvider()
 * @see isac.wsdialog.WorksheetDialog
 * @author Robert Koenighofer
 */
public interface IContextProvider {

	public enum ContextType {
		CONTEXT_EXAMPLES,
		CONTEXT_THEORIES,
		CONTEXT_PROBLEMS,
		CONTEXT_METHODS; 
		
		public String toString() {
			switch( this ) {
			case CONTEXT_EXAMPLES:
				return "exp";
			case CONTEXT_METHODS:
				return "met";
			case CONTEXT_PROBLEMS:
				return "pbl";
			case CONTEXT_THEORIES:
				return "thy";
			}
			return "";
		}
		
		public String toSMLString() {
			switch( this ) {
			case CONTEXT_EXAMPLES:
				return "Exp_";
			case CONTEXT_METHODS:
				return "Met_";
			case CONTEXT_PROBLEMS:
				return "Pbl_";
			case CONTEXT_THEORIES:
				return "Thy_";
			}
			return "";			
		}
	}
	
	/**
	 * This method initializes the Context of one of the browser
	 * dialoges.  The returned Context object contains the
	 * KEStoreKey of the element of the knowledge base which belongs
	 * to the context of the active WorksheetDialog and which has to
	 * be shown in the knowledge browser. This method is always called
	 * if the user opens a knowledge browser.
	 * 
	 * @param context just used to overload the method instead of switch-case
	 * @return the Context containing the KEStoreKey.
	 * @throws RemoteException if remote object is not available
	 * @see isac.util.formulae.ContextMethod
	 * 
	 */
	public Context initContext( ContextType context_type );
	
	
	/**
	 * This method reads out the Context of a selected item of the
	 * knowledge hierarchy. The HierarchyKey is unique, so no additional
	 * information is required to determine to which kind of knowledge
	 * (methods, problems or theories) the selected item belongs.
	 * FIXXME: more details?, what does the context contain exactly?
	 *  
	 * @param key the unique key of the selected element of the hierarchy
	 * @return the context object
	 * @throws RemoteException if remote object is not available
	 * @see isac.util.formulae.Context
	 * 
	 */
	public Context checkContext(Context context);
	
	/**
	 * This method causes the selected context to be accepted for the
	 * active worksheet.
	 * FIXXME: more details?, what does the context contain exactly?
	 * 
	 * @param context the context to accept for the active WorksheetDialog
	 * @throws RemoteException if remote object is not available
	 * @see isac.util.formulae.Context
	 * 
	 */
	public void setContext(Context context);
	
	/***
	 * This method services the Try Refine command
	 * 
	 */
	public Context refineContext( Context context );
	
	/***
	 * This method queries the state of the worksheet. 
	 * Currently the state means whether the CalcHeadPanel is shown or not.
	 * 
	 * @return the state of the worksheet
	 */
	public WorksheetState getWorksheetState();
	
}
