/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.wsdialog;

import java.rmi.Remote;
import java.rmi.RemoteException;

import isac.bridge.CalcIterator;
import isac.interfaces.*;
import isac.useractions.IUIElement;
import isac.useractions.IUserAction;

/**
 * @author Alan Krempler
 *
 */
/**
 * @author Alan Krempler
 * 
 * The interface of the Dialog Guide as seen from the Presentation Layer
 *  
 */
public interface IWorksheetDialog extends IUIElement, /* IToCalc, */Remote {

    /**
     * To express that the user started a calculation from scratch
     */
    public static final int STARTFROM_NEW = 1;

    /**
     * To express that the user started a calculation from scratch WN050415 this
     * decision is triggered by the CalcHead.viewstyle
     */
    //public static final int STARTFROM_EMPTY_CALCHEAD = 2;
    /**
     * To express that the user started a calculation from the example
     * collection
     */
    public static final int STARTFROM_EXAMPLE = 3;

    /**
     * To express that the user started a calculation from the hierarchy_ of
     * problems
     */
    public static final int STARTFROM_PROBLEM = 4;

    public static final int STARTFROM_METHOD = 5;
    
    public static final int STARTFROM_THEORY = 6;

    /**
     * The Dialog is not doing anything at the moment
     */
    public static final int DIALOGPHASE_IDLE = 10;

    /**
     * The Dialog guides the user through the specification of a problem;
     * WN050420 in this phase the CalcHeadPanel is presented to the User; the
     * phase is entered iff:
     * 
     * (in_SPECIFY_1) the Worksheet does CalcIterator#getFormula while
     * CalcIterator.onCalcHead _AND_ the WorksheetDialog agrees due to
     * UserSettings
     * 
     * (in_SPECIFY_2) the Worksheets activeFormula is on a headline of a
     * CalcHead and the User selects the menuitem 'CalcHead'
     * 
     * This phase is finished iff:
     * 
     * (out_SPECIFY_1) the User closes the CalcHeadPanel (which requires a
     * 'complete' CalcHead)
     * 
     * (out_SPECIFY_2) the Worksheet does CalcTree#replaceFormula at a Position
     * the active CalcHeadPanel depends on (this allows to escape from an
     * 'incomplete' subproblem) TODO
     */
    public static final int DIALOGPHASE_SPECIFY = 11;

    /**
     * The Dialog guides the user to the solution of a problem
     * 
     * WN050420 this phase is entered iff:
     * 
     * (in_SOLVE_0) a new Worksheet is opened; this is done immediately followed
     * by DIALOGPHASE_SPECIFY case (I1) thus commonly starting a calculation
     * with the DIALOGPHASE_SPECIFY
     * 
     * (in_SOLVE_1)=(out_SPECIFY_1)
     * 
     * (in_SOLVE_2)=(out_SPECIFY_2)
     * 
     * This phase is finished iff:
     * 
     * (out_SOLVE_1)=(in_SPECIFY_1)
     * 
     * (out_SOLVE_2)=(in_SPECIFY_2)
     */
    public static final int DIALOGPHASE_SOLVE = 12;

    /**
     * The Dialog guides the user through the specification of a subproblem
     * 
     * @deprecated WN0504 since there is no related description
     */
    public static final int DIALOGPHASE_SPECIFY_SUBPROBLEM = 3;

    /**
     * Start the a calculation by providing initial values to the Dialog On
     * success, the Dialog is in DIALOGPHASE_SOLVE
     * 
     * @param user_id
     *            The user FIXME.WN050808 what for a user ??? delete !!!
     * @param f
     *            Formalization from the example collection or null
     * @param started_from
     *            STARTFROM_*
     * @return an empty CalcTree
     * 
     * @throws RemoteException
     * 
     * @see isac.util.Formalization
     */
    public IToCalc getCalcTree()
            throws RemoteException;

    /**
     * Start the a calculation by providing initial values to the Dialog On
     * success, the Dialog is in DIALOGPHASE_SOLVE
     */
    public void startCalculate() throws RemoteException;

    /**
     * Start the specification phase by providing initial values to the Dialog
     * On success, the Dialog is in DIALOGPHASE_SPECIFY
     * 
     * @see isac.wsdialog.IWorksheetDialog#DIALOGPHASE_SPECIFY *
     * @throws Exception
     *             because working within WorksheetDialog FIXXXME.WN050704
     *             should be local to WorksheetDialog !!!
     */
    //CalcHead startSpecifying(CalcHead c) throws RemoteException;//WN050421
    // OLD
    //public CalcHead startSpecifying() throws RemoteException;//WN050421 NEW
    /**
     * Start the solving phase. On success, the Dialog is in DIALOGPHASE_SOLVE
     * 
     * @throws Exception
     *             because working within WorksheetDialog
     */
    //public void startSolving() throws Exception;//WN050421 NEW
    //public void startSolving(CalcHead c) throws Exception;//WN050421 OLD
    /**
     * Register the UI to be controlled by the Dialog. There is only one such
     * UI, any already-registered UI will be detached.
     * 
     * @param listener
     *            UI which is giong to listen to the Dialog
     * @return true on success, flase othewise
     * @throws RemoteException
     */
    public boolean registerUIControlListener(IToGUI listener)
            throws RemoteException;

    /**
     * Notify the Dialog about events or requests triggered by the user
     * 
     * @param action
     *            What's new from the UI
     * @return true, if the event is being processed, false if processing is
     *         denied or impossible
     * @throws RemoteException
     * @throws DialogProtocolException
     */
    public boolean notifyUserAction(IUserAction action) throws RemoteException,
            DialogProtocolException;

    /**
     * Notify the Dialog if the calculation on the worksheet is totaly shown
     * 
     * ?should use a parameter to identify the calcChangedEvent??
     * 
     * @return true, if the event is being processed, false if processing is
     *         denied or impossible
     * @throws RemoteException
     */
    public void notifyGUIReady() throws RemoteException;

    /**
     * Register a new listener who wants to be notified of changes in the data.
     * The listener has to implement the IToUser interface.
     * 
     * @param listener
     *            New listener to be registered.
     * @return false on success, true otherwise.
     * 
     * WN050429 copied from IToCalc upon removing 'WorksheetDialog implements
     * IToCalc' redesign interfaces ?
     */
    public boolean addDataChangeListener(IToUser listener)
            throws RemoteException;

    /**
     * create an <code>ICalcIterator</code> for use in the
     * <code>Worksheet</code>
     * 
     * @param <code>CalcIterator</code> to be translated into a
     *            <code>DialogIterator</code>
     * @param <code>WorksheetDialog</code> which does the transation
     * 
     * @see isac.wsdialog.IToCalc#iterator()
     */
    public ICalcIterator iterator(CalcIterator it, IWorksheetDialog dg)
            throws RemoteException;

    /**
     * Returns the id of the Worksheet (which is actually the session_id!!!).
     * This is required by the ProblemBrowser to connect to the right CalcTree
     * to instantiate problems.
     * 
     * For use at generation of the Worksheet _ONLY_; thus a _VERY_ questionable
     * design descision, anyway !
     * 
     * @deprecated in favour of Worksheet-msg to Dialog's if brought in front
     */
    public String getWorksheetID() throws RemoteException;
}